-- This is written in Haskell.
{--
JVM-Bridge -- bridge from FP languages and others to the Java VM
Copyright (C) 2001 Ashley Yakeley <ashley@semantic.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--}

module BridgeThread where
	{
	import Class_java_lang_Thread;
	import Class_java_lang_Object;
	import BridgeCallback;
	import LibLayer;
	import TypedLayer;
	import JavaLayer;
	import BasicLayer;
	
	data Class_JHaskellThread;
	instance IsJavaClassMarker Class_JHaskellThread where
		{
		cName Type = toJavaString "org/semantic/jvmbridge/HaskellThread";
		};
	instance SubJavaClassMarker Class_Jjava_lang_Thread	Class_JHaskellThread;
	instance SubJavaClassMarker Class_Jjava_lang_Object	Class_JHaskellThread;

	type JHaskellThread = ObjectRef Class_JHaskellThread;
	
	class (HasPreload_JThread m) => HasRunnableClass m where
		{
		getRunnableClass :: m JClass;
		};
	
	makeRunnableClass :: (HasPreload_JThread m) => Jjava_lang_ClassLoader -> m JClass;
	makeRunnableClass loader = defineCallbackClass loader (Type :: Type JHaskellThread) (Type :: Type JThread) []
		[]
		[
		MkMemberNameType (toJavaString "run") (MkFunctionType [] MkVoidType)
		]
		;
	
	instance (HasRunnableClass m) => MonadThread m where
		{
		fork f = do
			{
			(thread::JThread) <- forkThread f;
			return ();
			};
		yield = yield_JThread ();
		sleep ns = sleep_JThread_Jlong_Jint (fromIntegral (div ns 100000),fromIntegral (mod ns 1000000));
		};
	
	instance (HasRunnableClass m,HasPreload_JObject m) => ThreadLock m JObject where
		{
		newLock = new_JObject ();
		synchronized = tlSynchronized;
		wait lock = wait_JObject lock ();
		notify lock = notify_JObject lock ();
		notifyAll lock = notifyAll_JObject lock ();
		};
	
	instance (HasRunnableClass m) => ThreadControl m JThread where
		{
		forkThread foo = do
			{
			htClass <- getRunnableClass;
			fooVal	<- makeCallback (\(thread::JHaskellThread) () -> foo);
			thread <- newObject htClass (fooVal);
			start_JThread thread ();
			return thread;
			};
		thisThread = currentThread_JThread ();
		wakeThread thread = interrupt_JThread thread ();
		joinThread thread = join_JThread thread ();
		};
	}
