-- This is written in Haskell.
{--
JVM-Bridge -- bridge from FP languages and others to the Java VM
Copyright (C) 2001 Ashley Yakeley <ashley@semantic.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--}

module ContextMonad where
	{
	import LiftedMonad;
	
	data (Monad m) => ContextMonad c m a = MkContextMonad (c -> m a);
	
	getContext :: (Monad m) => ContextMonad c m c;
	getContext = MkContextMonad (\c -> return c);

	doContext :: (Monad m) => ContextMonad c m a -> c -> m a;
	doContext (MkContextMonad p) = p;

	class
		(
		Monad m
		) =>
	 ContextFailable m c where
		{
		contextFail :: String -> ContextMonad c m a;
		contextFail s = MkContextMonad (\c -> fail s);
		};

	instance
		(
		Monad m,
		ContextFailable m c
		) =>
	 Monad (ContextMonad c m) where
		{
		(MkContextMonad p) >>= qf = 
			MkContextMonad (\c -> (p c) >>= (\a -> doContext (qf a) c));

		return a = MkContextMonad (\c -> return a);

		fail = contextFail;
		};

	instance
		(
		ContextFailable m c,
		SemiLiftedMonad l m
		) =>
	 SemiLiftedMonad l (ContextMonad c m) where
		{
		call f = MkContextMonad (\c -> call f);
		};

	instance
		(
		ContextFailable m c,
		LiftedMonad l m
		) =>
	 LiftedMonad l (ContextMonad c m) where
		{
		callWithMap f = do
			{
			c <- getContext;
			MkContextMonad (\c -> callWithMap (\map -> (f (map . (\m -> doContext m c)))));
			};
		};
	}
