-- This is written in Haskell.
{--
JVM-Bridge -- bridge from FP languages and others to the Java VM
Copyright (C) 2001 Ashley Yakeley <ashley@semantic.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--}

-- four-space tabs
module Main where
	{
	import Class_java_lang_System;
	import Class_java_lang_String;
	import ShowFrame_JVM;
	import JVMBridge;
	import Control.Concurrent;

	-- define our own Object type
	data Class_MyObject;
	instance IsJavaClassMarker Class_MyObject where
		{
		cName Type = toJavaString "org/semantic/jvmbridge/example/MyObject";
		};
	instance SubJavaClassMarker Class_Jjava_lang_Object		Class_MyObject;
	type MyObject = ObjectRef Class_MyObject;

	-- the callback function written out explicitly
	callbackFunction :: MVar () -> () -> JVM Jvoid;
	callbackFunction mvar () = do
		{
		callIO (putStrLn ("callbackFunction"));
		callIO (putMVar mvar ());
		};

	instance HasRunnableClass JVM where
		{
		getRunnableClass = do
			{		
			loader	<- getStandardClassLoader;
			makeRunnableClass loader;
			};
		};

	main :: IO ();
	main = run (do
		{	
		loader	<- getStandardClassLoader;									-- defineCallbackClass needs this
		hcClass <- defineCallbackClass loader (Type :: Type MyObject) (Type :: Type Jjava_lang_Object)
			[]																-- no interfaces
			[]																-- no constructor arguments other than methodvals
			[MkMemberNameType												-- just one overridden method
				(toJavaString "foo")										-- called 'foo'
				(MkFunctionType
					[]														-- empty argument list
					MkVoidType												-- void return type
				)
			]
			;

		mvar <- callIO newEmptyMVar;

		callbackFunctionVal	<- makeStaticCallback (callbackFunction mvar);	-- convert the method to a 'val' that can be passed to a constructor

		(myObject :: MyObject) <- newObject hcClass (callbackFunctionVal);	-- create a new MyObject
		
		foo <- getMethod hcClass (toJavaString "foo");

		callIO (putStrLn ("H+ main foo call"));
		() <- foo myObject ();
		callIO (putStrLn ("H- main foo call"));

		callIO (putStrLn ("H+ fork"));
		fork (do
			{
			callIO (putStrLn ("H+ thread foo call"));
			() <- foo myObject ();
			callIO (putStrLn ("H- thread foo call"));
			});
		callIO (putStrLn ("H- fork"));
		
		callIO (putStrLn ("Yielding to other Haskell threads..."));			-- tell user what's going on
		withProcsThread (yieldLoop (do										-- yields to other Java and Haskell threads
			{
			callIO (takeMVar mvar);
			return False;
			}));
		callIO (putStrLn ("Done!"));
		});
	}
