-- This is written in Haskell.
{--
JVM-Bridge -- bridge from FP languages and others to the Java VM
Copyright (C) 2001 Ashley Yakeley <ashley@semantic.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--}

module Invocation
	(
	runJVM,runTypedJVM,
	runJVMReport
	) where
	{
	import TypedLayer;
	import VMLayer;
	import JavaLayer;
	import BasicLayer;
	import CForeign;
	import Foreign;

	foreign import ccall "JVMInvocation.h JVMBridge_CreateJavaVM" rawCreateJavaVM ::
		Ptr CString -> IO JavaVM;

	withCStringArray :: [String] -> ([CString] -> IO a) -> IO a;
	withCStringArray [] f = f [];
	withCStringArray (s:ss) f = 
	 withCString s (\c ->
	 withCStringArray ss (\cc ->
	 f (c:cc)
	 ));

	createJavaVM :: [String] -> IO JavaVM;
	createJavaVM classPath = do
		{
		vm <- withCStringArray classPath (\cc -> withArray (cc ++ [nullPtr]) rawCreateJavaVM);
		if (badVM vm) then (fail "couldn't create Java VM") else (return vm);
		};

	runJVM :: (IsJVMMonad m) => [String] -> m a -> IO a;
	runJVM classPath foo = do
		{
		vm <- createJavaVM classPath;
		runWithVM vm foo;
		};

	runTypedJVM :: (IsJVMMonad m) => Type (m a) -> [String] -> m a -> IO a;
	runTypedJVM Type = runJVM;

	runJVMReport :: (IsJVMMonad m) => [String] -> m () -> IO ();
	runJVMReport classPath foo = runJVM classPath (runReport foo);
	}
