-- This is written in Haskell.
{--
JVM-Bridge -- bridge from FP languages and others to the Java VM
Copyright (C) 2001 Ashley Yakeley <ashley@semantic.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--}

module JVMBoot
	(
	JVMBoot
	) where
	{
	import TypedLayer;
	import VMLayer;
	import JavaLayer;
	import BasicLayer;
	import MarshalArray;
	
	type JVMBoot = ContextMonad (JavaVM,VMEnv) IO;
	
	instance ContextFailable IO (JavaVM,VMEnv);

	getVM :: JVMBoot JavaVM;
	getVM = do
		{
		(vm,_) <- getContext;
		return vm;
		};

	getEnv :: JVMBoot VMEnv;
	getEnv = do
		{
		(_,env) <- getContext;
		return env;
		};

	instance IsJVMMonad JVMBoot where
		{
		callIOWithMapEnv f = do
			{
			env <- getEnv;
			let {?jvmenv = env;} in MkContextMonad (\c -> callIOWithMap (\map -> (vmCallChecked (f (map . (\m -> doContext m c))))));
			};

		callIOWithEnv foo = do
			{
			env <- getEnv;
			let {?jvmenv = env;} in callIO (vmCallChecked foo);
			};

		getRunWithEnv = do
			{
			vm <- getVM;
			return (\env ma -> doContext ma (vm,env));
			};

		runWithVMAndEnv vm ma = doContext ma (vm,?jvmenv);
		};
	}
