-- This is written in Haskell.
{--
JVM-Bridge -- bridge from FP languages and others to the Java VM
Copyright (C) 2001 Ashley Yakeley <ashley@semantic.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--}

module JVMStandard
	(
	JVMStandard
	) where
	{
	import JVMLoadable;
	import JVMBoot;
	import TypedLayer;
	import VMLayer;
	import JavaLayer;
	import BasicLayer;
	import MarshalArray;
	
	data (IsJVMMonad m) => JVMStandardContext m = MkJVMStandardContext
		{
		cClassString				:: JClass,
		cMethodStringInit			:: [Jchar] -> m JString,
		cMethodStringToCharArray	:: JString -> () -> m [Jchar],
		cClassError					:: JClass,
		cMethodErrorInit			:: JString -> m JThrowable
		};
	
	instance (IsJVMMonad m) => IsJVMLoadable m (JVMStandardContext m) where
		{
		jvmLoad = do
			{
			classString				<- findClass (Type :: Type JString);
			methodStringInit		<- getMakeNewObject classString;
			methodStringToCharArray	<- getMethod classString (toJavaString "toCharArray");
			classError				<- findClassByName (toJavaString "java/lang/Error");
			methodErrorInit			<- getMakeNewObject classError;
			return (MkJVMStandardContext
				classString methodStringInit methodStringToCharArray
				classError methodErrorInit
				);
			};
		};
	
	type JVMStandard = ContextMonad (JVMStandardContext JVMBoot) JVMBoot;
	
	instance ContextFailable JVMBoot (JVMStandardContext JVMBoot) where
		{
		contextFail s = MkContextMonad (\c -> do
			{
			str <- cMethodStringInit c (toJavaString s);
			errObj <- cMethodErrorInit c str;
			throwException errObj;
			});
		};

	instance IsJVMMonad JVMStandard where
		{
		callIOWithEnv f = MkContextMonad (\c -> callIOWithEnv (f));

		callIOWithMapEnv f = MkContextMonad (\c -> callIOWithMapEnv (\map -> ((f) (map . (\m -> doContext m c)))));

		getRunWithEnv = MkContextMonad (\c -> do
			{
			runWithEnv <- getRunWithEnv;
			return (\env (MkContextMonad cma) -> runWithEnv env (cma c));
			});

		runWithVMAndEnv vm (MkContextMonad cma) = runWithVMAndEnv vm (do
			{
			context <- jvmLoad;
			cma context;
			});
		};
{--	
	instance MonadThread JVMStandard where
		{
		fork	:: m () -> m ();
		fork foo = do
			{
			vm <- getVM;
			
			};
		yield	:: m ();
		sleep	:: Integer -> m ();	-- nanoseconds
		};
--}
	}
