-- This is written in Haskell.
{--
JVM-Bridge -- bridge from FP languages and others to the Java VM
Copyright (C) 2001 Ashley Yakeley <ashley@semantic.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--}

module Main where
	{
	import JVMBoot;
	import Invocation;
	import TypedLayer;
	import JavaLayer;
	import BasicLayer;
	import System;
	import Bits;
	import IO;
	import System.Exit;


	data JObjectClass;
	instance IsJavaClassMarker JObjectClass where
		{cName Type = toJavaString "java/lang/Object";};
	type JObject = ObjectRef JObjectClass;
	class IsJObject c where
		{toJObject :: c -> JObject;};
	instance IsJObject JObject where
		{toJObject = castTLRef;};

	instance IsJObject JClass where
		{toJObject = castTLRef;};

	data JReaderClass;
	instance IsJavaClassMarker JReaderClass where
		{cName Type = toJavaString "java/io/Reader";};
	type JReader = ObjectRef JReaderClass;
	class IsJReader c where
		{toJReader :: c -> JReader;};
	instance IsJReader JReader where
		{toJReader = castTLRef;};
	instance IsJObject JReader where
		{toJObject = castTLRef;};

	data JBufferedReaderClass;
	instance IsJavaClassMarker JBufferedReaderClass where
		{cName Type = toJavaString "java/io/BufferedReader";};
	type JBufferedReader = ObjectRef JBufferedReaderClass;
	instance IsJReader JBufferedReader where
		{toJReader = castTLRef;};
	instance IsJObject JBufferedReader where
		{toJObject = castTLRef;};

	data JInputStreamReaderClass;
	instance IsJavaClassMarker JInputStreamReaderClass where
		{cName Type = toJavaString "java/io/InputStreamReader";};
	type JInputStreamReader = ObjectRef JInputStreamReaderClass;
	instance IsJReader JInputStreamReader where
		{toJReader = castTLRef;};
	instance IsJObject JInputStreamReader where
		{toJObject = castTLRef;};

	data JInputStreamClass;
	instance IsJavaClassMarker JInputStreamClass where
		{cName Type = toJavaString "java/io/InputStream";};
	type JInputStream = ObjectRef JInputStreamClass;
	instance IsJObject JInputStream where
		{toJObject = castTLRef;};

	data JPrintStreamClass;
	instance IsJavaClassMarker JPrintStreamClass where
		{cName Type = toJavaString "java/io/PrintStream";};
	type JPrintStream = ObjectRef JPrintStreamClass;
	instance IsJObject JPrintStream where
		{toJObject = castTLRef;};

	data JSystemClass;
	instance IsJavaClassMarker JSystemClass where
		{cName Type = toJavaString "java/lang/System";};
	type JSystem = ObjectRef JSystemClass;
	instance IsJObject JSystem where
		{toJObject = castTLRef;};

	data JMemberClass;
	instance IsJavaClassMarker JMemberClass where
		{cName Type = toJavaString "java/lang/reflect/Member";};
	type JMember = ObjectRef JMemberClass;
	class IsJMember c where
		{toJMember :: c -> JMember;};
	instance IsJMember JMember where
		{toJMember = castTLRef;};
	instance IsJObject JMember where
		{toJObject = castTLRef;};

	data JMethodClass;
	instance IsJavaClassMarker JMethodClass where
		{cName Type = toJavaString "java/lang/reflect/Method";};
	type JMethod = ObjectRef JMethodClass;
	instance IsJMember JMethod where
		{toJMember = castTLRef;};
	instance IsJObject JMethod where
		{toJObject = castTLRef;};

	data JConstructorClass;
	instance IsJavaClassMarker JConstructorClass where
		{cName Type = toJavaString "java/lang/reflect/Constructor";};
	type JConstructor = ObjectRef JConstructorClass;
	instance IsJMember JConstructor where
		{toJMember = castTLRef;};
	instance IsJObject JConstructor where
		{toJObject = castTLRef;};

	data JFieldClass;
	instance IsJavaClassMarker JFieldClass where
		{cName Type = toJavaString "java/lang/reflect/Field";};
	type JField = ObjectRef JFieldClass;
	instance IsJMember JField where
		{toJMember = castTLRef;};
	instance IsJObject JField where
		{toJObject = castTLRef;};


	type JVM = JVMBoot;


	runJVMOnArgClasspath :: (IsJVMMonad m) => m () -> IO ();
	runJVMOnArgClasspath foo = catch (do
		{
		classpath <- getArgs;
		runJVMReport classpath foo;
		})
		(\ex -> (do
		{
		putStrLn ("caught exception: " ++ (show ex));
		exitFailure;
		}));

	forAny :: (Monad m) => [a] -> (a -> m Bool) -> m Bool;
	forAny [] is = return False;
	forAny (a:as) is = do
		{
		isA <- is a;
		if isA then (return True) else (forAny as is);
		};

	foreachgen :: (Monad m) => c -> (b -> c -> m c) -> [a] -> (a -> m b) -> m c;
	foreachgen nothing gather [] f = return nothing;
	foreachgen nothing gather (a:as) f = do
		{
		b <- f a;
		c <- foreachgen nothing gather as f;
		gather b c;
		};

	foreach :: (Monad m) => [a] -> (a -> m ()) -> m ();
	foreach = foreachgen () (\() () -> return ());

	foreachres :: (Monad m) => [a] -> (a -> m b) -> m [b];
	foreachres = foreachgen [] (\b c -> return (b:c));

	foreachcollect :: (Monad m) => [a] -> (a -> m [b]) -> m [b];
	foreachcollect = foreachgen [] (\b c -> return (b ++ c));

	mergeOne :: (Monad m) => (a -> a -> m Bool) -> [a] -> a -> m [a];
	mergeOne eqF pl q = do
		{
		has <- forAny pl (\p -> eqF p q);
		return (if has
		 then pl
		 else (q:pl)
			);
		};

	merge :: (Monad m) => (a -> a -> m Bool) -> [a] -> [a] -> m [a];
	merge eqF pl [] = return pl;
	merge eqF pl (q:qs) = do
		{
		has <- forAny pl (\p -> eqF p q);
		if has
		 then (merge eqF pl qs)
		 else do
			{
			pq <- merge eqF pl qs;
			return (q:pq);
			};
		};

	foreachmerge :: (Monad m) => (b -> b -> m Bool) -> [a] -> (a -> m [b]) -> m [b];
	foreachmerge eqF = foreachgen [] (merge eqF);

	isWS :: Jchar -> Bool;
	isWS c = c <= 32;

	firstNWS :: JavaString -> (JavaString,JavaString);
	firstNWS [] = ([],[]);
	firstNWS (c:cs) | isWS c = ([],cs);
	firstNWS (c:cs) = let {(f,r) = firstNWS cs} in (c:f,r);

	trimWS :: JavaString -> JavaString;
	trimWS [] = [];
	trimWS (c:cs) | isWS c = trimWS cs;
	trimWS s = s;

	splitWS :: JavaString -> [JavaString];
	splitWS s = case firstNWS (trimWS s) of
		{
		([],_) -> [];
		(f,r) -> f:(splitWS r);
		};

	slashToDot :: Jchar -> Jchar;
	slashToDot 47 = 46;
	slashToDot c = c;

	dotToSlash :: Jchar -> Jchar;
	dotToSlash 46 = 47;
	dotToSlash c = c;

	main :: IO ();
	main = runJVMOnArgClasspath (do
		{
		objectClass				<- findClass (Type :: Type JObject);
		(equalsMethod :: JObject -> (JObject) -> JVM Jboolean)
								<- getMethod objectClass (toJavaString "equals");

		systemClass				<- findClass (Type :: Type JSystem);
		(inRef :: Ref JVM JInputStream)
								<- getStaticField systemClass (toJavaString "in");
		(outRef :: Ref JVM JPrintStream)
								<- getStaticField systemClass (toJavaString "out");

		stringClass				<- findClass (Type :: Type JString);
		(toCharArray :: JString -> () -> JVM [Jchar])
								<- getMethod stringClass (toJavaString "toCharArray");
		(makeNewString :: ([Jchar]) -> JVM JString)
								<- getMakeNewObject stringClass;
		
		readerClass				<- findClass (Type :: Type JReader);
		(close :: JReader -> () -> JVM ())
								<- getMethod readerClass (toJavaString "close");

		fileInputStreamClass	<- findClassByName (toJavaString "java/io/FileInputStream");
		(makeNewFileInputStream :: JString -> JVM JInputStream)
								<- getMakeNewObject fileInputStreamClass;

		inputStreamReaderClass	<- findClass (Type :: Type JInputStreamReader);
		(makeNewInputStreamReader :: JInputStream -> JVM JInputStreamReader)
								<- getMakeNewObject inputStreamReaderClass;

		bufferedReaderClass		<- findClass (Type :: Type JBufferedReader);
		(makeNewBufferedReader :: JReader -> JVM JBufferedReader)
								<- getMakeNewObject bufferedReaderClass;
		(readLine :: JBufferedReader -> () -> JVM JString)
								<- getMethod bufferedReaderClass (toJavaString "readLine");

		printStreamClass		<- findClass (Type :: Type JPrintStream);
		(println :: JPrintStream -> ([Jchar]) -> JVM Jvoid)
								<- getMethod printStreamClass (toJavaString "println");		

		classClass				<- findClass (Type :: Type JClass);
		(forName :: (JString) -> JVM JClass)
								<- getStaticMethod classClass (toJavaString "forName");		
		(getName :: JClass -> () -> JVM JString)
								<- getMethod classClass (toJavaString "getName");
		(getClassModifiers :: JClass -> () -> JVM Jint)
								<- getMethod classClass (toJavaString "getModifiers");
		(getInterfaces :: JClass -> () -> JVM [JClass])
								<- getMethod classClass (toJavaString "getInterfaces");
		(getSuperclass :: JClass -> () -> JVM JClass)
								<- getMethod classClass (toJavaString "getSuperclass");
		(getDeclaredFields :: JClass -> () -> JVM [JField])
								<- getMethod classClass (toJavaString "getDeclaredFields");
		(getDeclaredMethods :: JClass -> () -> JVM [JMethod])
								<- getMethod classClass (toJavaString "getDeclaredMethods");
		(getDeclaredConstructors :: JClass -> () -> JVM [JConstructor])
								<- getMethod classClass (toJavaString "getDeclaredConstructors");
		(getDeclaredMethod :: JClass -> (JString,[JClass]) -> JVM JMethod)
								<- getMethod classClass (toJavaString "getDeclaredMethod");

		constructorClass		<- findClass (Type :: Type JConstructor);
		(constructorgetParameterTypes :: JConstructor -> () -> JVM [JClass])
								<- getMethod constructorClass (toJavaString "getParameterTypes");
		
		memberClass				<- findClass (Type :: Type JMember);
		(membergetName :: JMember -> () -> JVM JString)
								<- getMethod memberClass (toJavaString "getName");
		(getModifiers :: JMember -> () -> JVM Jint)
								<- getMethod memberClass (toJavaString "getModifiers");
		
		fieldClass				<- findClass (Type :: Type JField);
		(getType :: JField -> () -> JVM JClass)
								<- getMethod fieldClass (toJavaString "getType");
		
		methodClass				<- findClass (Type :: Type JMethod);
		(getReturnType :: JMethod -> () -> JVM JClass)
								<- getMethod methodClass (toJavaString "getReturnType");
		(getParameterTypes :: JMethod -> () -> JVM [JClass])
								<- getMethod methodClass (toJavaString "getParameterTypes");
		
		
		classesIS	<- get inRef;
		classesISR	<- makeNewInputStreamReader classesIS;
		classesBR	<- makeNewBufferedReader (toJReader classesISR);
		
		outStream	<- get outRef;
		
		let
			{
			outputLine :: JavaString -> JVM ();
--			outputLine s = return ();
--			outputLine s = println outStream s;
--			outputLine s = callIO (putStrLn (showJavaString s));
			outputLine s = callIO (do
				{
				putStrLn (showJavaString s);
				hFlush stdout;
				});
	
			commaList :: [JavaString] -> JavaString;
			commaList [] = [];
			commaList (x:[]) = x;
			commaList (x:xs) = x ++ (toJavaString ",") ++ (commaList xs);
			
			parameterList :: [JavaString] -> JavaString;
			parameterList ss = (toJavaString "(") ++ (commaList ss) ++ (toJavaString ")");
			
			-- m4-style quoting
			enquote :: JavaString -> JavaString;
			enquote x = (toJavaString "`") ++ x ++ (toJavaString "'");

			mkCmd :: String -> [JavaString] -> JavaString;
			mkCmd cmd ss = (toJavaString cmd) ++ (parameterList (fmap enquote ss));
	
			outputCmd :: String -> [JavaString] -> JVM ();
			outputCmd cmd ss = outputLine (mkCmd cmd ss);

			equals :: (IsJObject a,IsJObject b) => a -> (b) -> JVM Jboolean;
			equals a b = equalsMethod (toJObject a) (toJObject b);

			getNameArray :: JClass -> JVM JavaString;
			getNameArray c = do
				{
				str <- getName c ();
				rawName <- toCharArray str ();
				return (fixClassName rawName);
				} where
				{
				fixClassName :: JavaString -> JavaString;
				fixClassName (f:s) | f == (toJChar '[')		= typeNameFromSig (f:s) where
					{
					typeNameFromSig :: JavaString -> JavaString;
					typeNameFromSig (f:[])	| f == (toJChar 'V')	= (toJavaString "void");
					typeNameFromSig (f:[])	| f == (toJChar 'Z')	= (toJavaString "boolean");
					typeNameFromSig (f:[])	| f == (toJChar 'B')	= (toJavaString "byte");
					typeNameFromSig (f:[])	| f == (toJChar 'C')	= (toJavaString "char");
					typeNameFromSig (f:[])	| f == (toJChar 'S')	= (toJavaString "short");
					typeNameFromSig (f:[])	| f == (toJChar 'I')	= (toJavaString "int");
					typeNameFromSig (f:[])	| f == (toJChar 'J')	= (toJavaString "long");
					typeNameFromSig (f:[])	| f == (toJChar 'F')	= (toJavaString "float");
					typeNameFromSig (f:[])	| f == (toJChar 'D')	= (toJavaString "double");
					typeNameFromSig (f:s)	| (f == (toJChar 'L')) && ((last s) == (toJChar ';'))	= take ((length s) - 1) s;
					typeNameFromSig (f:s)	| f == (toJChar '[')	= (toJavaString "[") ++ (typeNameFromSig s) ++ (toJavaString "]");
					};
				fixClassName s = s;
				};
			
			foundInClasses :: [JClass] -> JString -> [JClass] -> JVM Bool;
			foundInClasses classes mName mTypes = forAny classes (\cl -> foundInClass cl mName mTypes) where
				{				
				foundInClass :: JClass -> JString -> [JClass] -> JVM Bool;
				foundInClass cl mName mTypes = do
					{
					theMethod <- catchJThrowable (getDeclaredMethod cl (mName,mTypes)) (\x -> getNothing);
					noMethod <- getIsNothing theMethod;
					return (not noMethod);
					};
				};
			
			outputClass :: JavaString -> JVM ();
			outputClass className = do
				{
				theClass <- catchJThrowable (findClassByName (fmap dotToSlash className)) (\x -> getNothing);
				noClass <- getIsNothing theClass;
				if (noClass)
				 then do
				 	{
					outputLine [];
					outputLine ((toJavaString "dnl ") ++ className ++ (toJavaString " not found"));
					callIO (do
						{
						hPutStrLn stderr ("ShowClasses: " ++ (showJavaString className) ++ " not found");
						hFlush stderr;
--						exitFailure;
						});
				 	}
				 else do
					{
					modifiers <- getClassModifiers theClass ();
					-- public classes only
					if (modifiers .&. 1 > 0) then do
						{
						name <- getNameArray theClass;
						outputLine [];
						outputLine ((toJavaString "dnl ") ++ name);
						outputCmd "sc_classname" [name];

						-- superclasses and interfaces
						supers <- let
							{
							newInterfaces :: [JClass] -> JClass -> JVM [JClass];
							newInterfaces intfs aClass = do
								{
								interfaces <- getInterfaces aClass ();
								foreachmerge equals interfaces (\iClass -> do
									{
									already <- forAny intfs (\intf -> equals intf iClass);
									if already
									 then (return [])
									 else do
										{
										iSupers <- newInterfaces intfs iClass;
										return (iClass:iSupers);
										};
									});
								};
							addSupers :: [JClass] -> JClass -> JVM [JClass];
							addSupers intfs aClass = do
								{
								interfaces <- newInterfaces intfs aClass;
								sClass <- getSuperclass aClass ();
								noSClass <- getIsNothing sClass;
								if noSClass
								 then (return (intfs ++ interfaces))
								 else do
									{
									sSupers <- addSupers (intfs ++ interfaces) sClass;
									return (sClass:sSupers);
									};
								};
							}
						in addSupers [] theClass;
						
						foreach supers (\cl -> do
							{
							cName <- getNameArray cl;
							outputCmd "sc_subclass" [name,cName];
							});
						
						-- fields
						fields <- getDeclaredFields theClass ();
						foreach fields (\field -> do
							{
							modifiers <- getModifiers (toJMember field) ();
							-- public only
							if (modifiers .&. 1 > 0) then do
								{
								jname <- membergetName (toJMember field) ();
								name <- toCharArray jname ();
								ftype <- getType field ();
								ftname <- getNameArray ftype;
								outputCmd
								 (if (modifiers .&. 8 > 0) then "sc_staticfield" else "sc_field")
								 [className,name,ftname];
								}
							 else (return ());
							});
						
						-- methods
						methods <- getDeclaredMethods theClass ();
						foreach methods (\method -> do
							{
							modifiers <- getModifiers (toJMember method) ();
							-- public only
							if (modifiers .&. 1 > 0) then do
								{
								let {isStatic = modifiers .&. 8 > 0};
								methodName <- membergetName (toJMember method) ();
								methodPTypes <- getParameterTypes method ();
								inherited <- if (isStatic) then (return False) else (foundInClasses supers methodName methodPTypes);
								if (inherited)
								 then (return ())
								 else do
									{
									name <- toCharArray methodName ();
									mtype <- getReturnType method ();
									mtname <- getNameArray mtype;
									pnames <- foreachres methodPTypes getNameArray;
									outputCmd
									 (if isStatic then "sc_staticmethod" else "sc_method")
									 [className,name,mtname,parameterList pnames];
									};
								}
							 else (return ());
							});
						
						-- constructors
						constructors <- getDeclaredConstructors theClass ();
						foreach constructors (\method -> do
							{
							modifiers <- getModifiers (toJMember method) ();
							-- public only
							if (modifiers .&. 1 > 0) then do
								{
								let {isStatic = modifiers .&. 8 > 0};
								methodPTypes <- constructorgetParameterTypes method ();
								pnames <- foreachres methodPTypes getNameArray;
								outputCmd "sc_constructor" [className,parameterList pnames];
								}
							 else (return ());
							});

						outputCmd "sc_endclass" [name];
						}
					else callIO (do
						{
						hPutStrLn stderr ("ShowClasses: " ++ (showJavaString className) ++ " not public");
						hFlush stderr;
						});
					};
				};
			};
		while (do
			{
			jClassNamesLine <- readLine classesBR ();
			stop <- getIsNothing jClassNamesLine;
			if (stop)
			 then (return False)
			 else do
				{
				classNamesLine <- toCharArray jClassNamesLine ();
				mapM_ (outputClass . (fmap slashToDot)) (splitWS classNamesLine);
				return True;
				};
			});
		
		close (toJReader classesBR) ();

		return ();
		} :: JVM ());
	}
