-- This is written in Haskell.
{--
JVM-Bridge -- bridge from FP languages and others to the Java VM
Copyright (C) 2001 Ashley Yakeley <ashley@semantic.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--}

module TypedCallback where
	{
	import TypedObject;
	import TypedMethod;
	import TypedClass;
	import TypedTuple;
	import TypedPrimitive;
	import TypedReturnable;
	import TypedReference;
	import TypedValue;
	import TypedMonad;
	import TypedArgumentList;
	import VMLayer;
	import JavaLayer;
	import BasicLayer;
	import Control.Concurrent;
	import Control.Exception;

{--
	data CallbackClass t = MkCallbackClass JClass;

	defineCallbackClass :: (IsJVMMonad m,IsJVMArgumentList arg) =>
		ClassName -> ClassName -> Type arg -> [MethodNameType] -> IO JClass;
--}
	
	startExecuteFunction :: (IsJVMMonad m) => m ();
	startExecuteFunction = callIOWithEnv vmStartExecuteFunction; 

	tlDefineCallbackClass :: IsJVMMonad m => VMRef -> ClassName -> ClassName -> [ClassName] -> [ValueType] -> [MethodNameType] -> m JClass;
	tlDefineCallbackClass loader classname superclassname interfaces superclassconstructor methods = do
		{
--		loader <- callIO getNothing;
		startExecuteFunction;
		callIOWithEnv (vmDefineCallbackClass loader classname superclassname interfaces superclassconstructor methods);
		findClassByName classname; -- this is curiously necessary
		};

	makeCallback :: (IsJVMMonad m,IsJVMReference c,IsJVMArgumentList a,IsJVMReturnable r) => (c -> a -> m r) -> m OpaqueAddress;
	makeCallback foo = tlMakeCallback (\vl -> do
		{
		c <- tlGetNthValueAs vl 0;
		(a,_) <- tlExtractValues vl 1;
		foo c a;
		});

	makeStaticCallback :: (IsJVMMonad m,IsJVMArgumentList a,IsJVMReturnable r) => (a -> m r) -> m OpaqueAddress;
	makeStaticCallback foo = tlMakeCallback (\vl -> do
		{
		(a,_) <- tlExtractValues vl 0;
		foo a;
		});
	
	getExecuteFunctionClass :: (IsJVMMonad m) => m JClass;
	getExecuteFunctionClass = do
		{
		startExecuteFunction; 
		findClassByName executeFunctionClassName;
		};
	
	getprocPending :: (IsJVMMonad m) => JClass -> m (() -> m Jboolean);
	getprocPending execClass = getStaticMethod execClass (toJavaString "procPending");
	
	getdoNextProc :: (IsJVMMonad m) => JClass -> m (() -> m ());
	getdoNextProc execClass = getStaticMethod execClass (toJavaString "doNextProc");
	
	-- Haskell fork, not Java VM fork
	tlForkIO :: (IsJVMMonad m) => m () -> m ThreadId;
	tlForkIO f = callIOWithMap (\map -> forkIO (map f));
	
	-- Haskell yield, not Java VM yield
	tlYield :: (IsJVMMonad m) => m ();
	tlYield = callIO Control.Concurrent.yield;
	
	runProcs :: (IsJVMMonad m) => m Jboolean -> m () -> m Bool -> m ();
	runProcs procPending doNextProc stopNow = do
		{
		stop <- stopNow;
		if (stop) then (return ()) else (do
			{
			pending <- procPending;
			if (pending) then (do
				{
				tlForkIO (do
					{
					result <- doNextProc;
					return result;
					});
				return ();
				}) else (return ());
			tlYield;
			runProcs procPending doNextProc stopNow; 
			});
		};
	
	-- returns the 'stop' proc.
	startProcsThread :: (IsJVMMonad m) => m (m ());
	startProcsThread = do
		{
		executeClass	<- getExecuteFunctionClass;
		procPending		<- getprocPending executeClass;
		doNextProc		<- getdoNextProc executeClass;
		stopVar			<- callIO newEmptyMVar;
		tlForkIO 
			(runProcs (procPending ()) (doNextProc ()) (callIO (isEmptyMVar stopVar) >>= (return .  not)));
		return (callIO (tryPutMVar stopVar ()) >> return ());
		};
	
	callIOWith3Maps	:: (LiftedMonad IO m) => (
		(m a -> IO a) -> 
		(m b -> IO b) -> 
		(m c -> IO c) -> 
		IO r) -> m r;
	callIOWith3Maps f = callIOWithMap (\map -> let
		{
		map1 ma = (map (ma >>= (return . Left . Left))) >>= (\(Left (Left a)) -> return a);
		map2 ma = (map (ma >>= (return . Left . Right))) >>= (\(Left (Right a)) -> return a);
		map3 ma = (map (ma >>= (return . Right))) >>= (\(Right a) -> return a);
		}
		in f map1 map2 map3);
	
	withProcsThread :: (IsJVMMonad m) => m a -> m a;
	withProcsThread f = callIOWith3Maps (\map1 map2 map3 -> bracket (map1 startProcsThread) map2 (const (map3 f)));
	
	tlYieldLoop :: (IsJVMMonad m) => (() -> m ()) -> m Bool -> m ();
	tlYieldLoop yieldMethod stopNow = do
		{
		while (do
			{
			stop <- stopNow;
			if (stop)
			 then (return True)
			 else (do
				{
				yieldMethod ();
				tlYield;
				return False;
				});
			});
		};
	}
