-- This is written in Haskell.
{--
JVM-Bridge -- bridge from FP languages and others to the Java VM
Copyright (C) 2001 Ashley Yakeley <ashley@semantic.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--}

module TypedClass
	(
	Class_JClass,
	JClass,
	findClassByName,
	findClass,
	getClass,
	isInstanceOfClass,
	isInstanceOf,
	SubJavaClassMarker(..)
	) where
	{
	import TypedObject;
	import TypedReference;
	import TypedMonad;
	import VMLayer;
	import JavaLayer;
	import BasicLayer;
	import MarshalArray;

	data Class_JClass;
	instance IsJavaClassMarker Class_JClass where
		{
		cName Type = toJavaString "java/lang/Class";
		};
	type JClass = ObjectRef Class_JClass;

	findClassByName :: (IsJVMMonad m) => ClassName -> m JClass;
	findClassByName className = do
		{
		classRef <- tlFindClassByName className;
		return (MkTLRef classRef);
		};
	
	findClass :: (IsJVMMonad m, IsJVMReference t) => Type t -> m JClass;
	findClass t = findClassByName (findClassName t);
	
	getClass :: (IsJVMMonad m, IsJVMReference t) => t -> m JClass;
	getClass obj = findClass (getType obj);
	
	isInstanceOfClass :: (IsJVMMonad m, IsJVMReference t) => JClass -> t -> m Bool;
	isInstanceOfClass (MkTLRef cl) obj = do
		{
		vmObj <- tlToRef obj;
		callIOWithEnv (vmIsInstanceOf vmObj cl);
		};
	
	isInstanceOf :: (IsJVMMonad m, IsJVMReference ct, IsJVMReference ot) => Type ct -> ot -> m Bool;
	isInstanceOf t obj = do
		{
		cl <- findClass t;
		isInstanceOfClass cl obj;
		};

	class (IsJavaClassMarker super,IsJavaClassMarker sub) => SubJavaClassMarker super sub where
		{
		objUpcast :: ObjectRef sub -> ObjectRef super;
		objUpcast sub = castTLRef sub;

--		objUpcast :: (IsJVMMonad m) => ObjectRef sub -> m (ObjectRef super);
--		objUpcast sub = return (castTLRef sub);

		objDowncast :: (IsJVMMonad m) => ObjectRef super -> m (Maybe (ObjectRef sub));
		objDowncast obj = do
			{
			is <- objIs (Type :: Type (ObjectRef sub)) obj;
			return (if is then (Just (castTLRef obj)) else Nothing);
			};

		objIs :: (IsJVMMonad m) => Type (ObjectRef sub) -> ObjectRef super -> m Bool;
		objIs = isInstanceOf;
		};

	instance
		(
		SubJavaClassMarker super sub
		) =>
	 IsA (ObjectRef super) (ObjectRef sub) where
		{
		convert	= objUpcast;
		};

	instance
		(
		IsJVMMonad m,
		SubJavaClassMarker super sub
		) =>
	 MonadIsA m (ObjectRef super) (ObjectRef sub) where
		{
		getConvert sub	= return (upcast sub);
		};

	instance
		(
		IsJVMMonad m,
		SubJavaClassMarker super sub
		) =>
	 MonadMaybeA m (ObjectRef sub) (ObjectRef super) where
		{
		getMaybeConvert	= objDowncast;
		getIs			= objIs;
		};

	instance
		(
		IsJVMMonad m,
		SubJavaClassMarker super sub
		) =>
	 MonadSubtype m (ObjectRef super) (ObjectRef sub);
	}
