-- This is written in Haskell.
{--
JVM-Bridge -- bridge from FP languages and others to the Java VM
Copyright (C) 2001 Ashley Yakeley <ashley@semantic.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--}

module TypedField
	(
	getStaticFieldByType,getFieldByType,
	getStaticField,getField
	) where
	{
	import TypedClass;
	import TypedObject;
	import TypedReference;
	import TypedValue;
	import TypedMonad;
	import VMLayer;
	import JavaLayer;
	import BasicLayer;
	import MarshalArray;

	tlGetStaticFieldID :: (IsJVMMonad m) => VMRef -> MemberName -> ValueType -> m VMStaticFieldID;
	tlGetStaticFieldID jClass name vtype = callIOWithEnv (vmGetStaticFieldID jClass name vtype);

	tlGetFieldID :: (IsJVMMonad m) => VMRef -> MemberName -> ValueType -> m VMFieldID;
	tlGetFieldID jClass name vtype = callIOWithEnv (vmGetFieldID jClass name vtype);

	getStaticFieldByType :: (IsJVMMonad m,IsJVMMonad mm,IsJVMValue v) =>
		JClass -> MemberName -> Type v -> m (Ref mm v);
	getStaticFieldByType (MkTLRef jClass) name vType = do
		{
		fieldID <- tlGetStaticFieldID jClass name (getValueType vType);
		return (MkRef
			{
			getRef = tlGetStaticField jClass fieldID,
			setRef = tlSetStaticField jClass fieldID
			});
		};

	getFieldByType :: (IsJVMMonad m,IsJVMMonad mm,IsJavaClassMarker c,IsJVMValue v) =>
		JClass -> MemberName -> Type v -> m (ObjectRef c -> Ref mm v);
	getFieldByType (MkTLRef jClass) name vType = do
		{
		fieldID <- tlGetFieldID jClass name (getValueType vType);
		return (\obj -> MkRef
			{
			getRef = do
				{
				rawObj <- tlToRef obj;
				tlGetField rawObj fieldID;
				},
			setRef = \val -> do
				{
				rawObj <- tlToRef obj;
				tlSetField rawObj fieldID val;
				}
			});
		};

	getStaticField :: (IsJVMMonad m,IsJVMMonad mm,IsJVMValue v) =>
		JClass -> MemberName -> m (Ref mm v);
	getStaticField cls name = getStaticFieldByType cls name Type;

	getField :: (IsJVMMonad m,IsJVMMonad mm,IsJavaClassMarker c,IsJVMValue v) =>
		JClass -> MemberName -> m (ObjectRef c -> Ref mm v);
	getField cls name = getFieldByType cls name Type where;
	}
