-- This is written in Haskell.
{--
JVM-Bridge -- bridge from FP languages and others to the Java VM
Copyright (C) 2001 Ashley Yakeley <ashley@semantic.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--}

module TypedLayer
	(
	module TypedClass,
	module TypedMonad,
	module TypedReturnable,
	module TypedValue,
	module TypedArgumentList,
	module TypedReference,
	module TypedObject,
	module TypedThrowable,
	module TypedTuple,
	module TypedString,
	module TypedArray,
	module TypedField,
	module TypedMethod,
	module TypedNewObject,
	module TypedThread,
	module TypedCallback,
	runJavaClass,reportThrowable,runReport
	) where
	{
	import TypedCallback;
	import TypedThread;
	import TypedNewObject;
	import TypedMethod;
	import TypedField;
	import TypedPrimitive;
	import TypedListArray;
	import TypedArray;
	import TypedString;
	import TypedTuple;
	import TypedThrowable;
	import TypedObject;
	import TypedReference;
	import TypedArgumentList;
	import TypedValue;
	import TypedReturnable;
	import TypedClass;
	import TypedMonad;
	import VMLayer;
	import JavaLayer;
	import BasicLayer;
	import Control.Monad.Error;

	runJavaClass :: (IsJVMMonad m) => ClassName -> [JString] -> m ();
	runJavaClass className args = do
		{
		mainClass <- findClassByName className;
		callStaticMethod mainClass (toJavaString "main") args;
		};
	
	reportThrowable :: (IsJVMMonad m) => JThrowable -> m ();
	reportThrowable ex = do
		{
		-- callIO (putStrLn "caught Java throwable");
		throwClass		<- findClass (Type::Type JThrowable);
		toStringMethod	<- getMethod throwClass (toJavaString "toString");
		jstr			<- toStringMethod ex ();
		str				<- getStringUTF jstr;
		callIO (putStrLn ("Java throwable: " ++ (showUTF8 str)));
		};
	
	runReport :: (IsJVMMonad m) => m () -> m ();
	runReport foo = (catchException (catchException foo reportThrowable) (\(ex :: JThrowable) -> (do
		{
		callIO (putStrLn "caught Java throwable in catch code!");
		})));
	}
