-- This is written in Haskell.
{--
JVM-Bridge -- bridge from FP languages and others to the Java VM
Copyright (C) 2001 Ashley Yakeley <ashley@semantic.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--}

module TypedListArray
	(
	toArray,fromArray
	) where
	{
	import TypedArray;
	import TypedReference;
	import TypedArgumentList;
	import TypedValue;
	import TypedReturnable;
	import TypedClass;
	import TypedMonad;
	import VMLayer;
	import JavaLayer;
	import BasicLayer;

	instance (IsJVMValue base) => IsJVMReturnable [base] where
		{
		getReturnableType t = MkValueType (getValueType t);
		tlCallStaticMethodReturning classObj method vlist = do
			{
			result <- callIOWithEnv (vmCallStaticRefMethod classObj method vlist);
			tlFromRef result;
			};
		tlCallMethodReturning obj method vlist = do
			{
			result <- callIOWithEnv (vmCallRefMethod obj method vlist);
			tlFromRef result;
			};
		tlCallNonvirtualMethodReturning obj classObj method vlist = do
			{
			result <- callIOWithEnv (vmCallNonvirtualRefMethod obj classObj method vlist);
			tlFromRef result;
			};
		tlMakeCallback foo = do
			{
			runWithEnv <- getRunWithEnv;
			callIOWithMap (\map -> vmMakeRefCallback (\vl -> do
				{
				ret <- runWithEnv ?jvmenv (foo vl);
				vmDestroyValueList vl;
				map (tlToRef ret);
				}));
			};
		};
	
	instance (IsJVMValue base) => IsJVMValue [base] where
		{
		getValueType Type = MkArrayType (getValueType (Type::Type base));
		tlAppendArgumentToValueList vlist list = do
			{
			obj <- tlToRef list;
			callIO (vmAddRefToValueList vlist obj);
			};
		tlGetNthValueAs vList i = do
			{
			obj <- callIOWithEnv (vmGetNthValueAsRef vList i);
			tlFromRef obj;
			};

		tlGetStaticField cls id = do
			{
			ref <- callIOWithEnv (vmGetStaticRefField cls id);
			tlFromRef ref;
			};

		tlSetStaticField cls id val = do
			{
			ref <- tlToRef val;
			callIOWithEnv (vmSetStaticRefField cls id ref);
			};

		tlGetField obj id = do
			{
			ref <- callIOWithEnv (vmGetRefField obj id);
			tlFromRef ref;
			};

		tlSetField obj id val = do
			{
			ref <- tlToRef val;
			callIOWithEnv (vmSetRefField obj id ref);
			};

		tlNewArray t size = do
			{
			elementClass <- tlFindClass t;
			nullRef <- callIO getNothing;
			callIOWithEnv (vmNewRefArray size elementClass nullRef);
			};

		tlGetArrayElement array i = do
			{
			elem <- tlGetArrayElement array i;
			fromArray elem;
			};

		tlGetArrayRegion array start len = if len <=0 then return [] else do
			{
			elem <- tlGetArrayElement array start;
			rest <- tlGetArrayRegion array (start + 1) (len - 1);
			return (elem:rest);
			};

		tlSetArrayElement array i list = do
			{
			elem <- toArray list;
			tlSetArrayElement array i elem;
			};

		tlSetArrayRegion array start [] = return ();
		tlSetArrayRegion array start (x:xs) = do
			{
			tlSetArrayElement array start x;
			tlSetArrayRegion array (start+1) xs;
			};
		};
		
	instance (IsJVMValue base) => IsJVMReference [base] where
		{
		findClassName t = valueTypeSig (getValueType t);
		tlToRef list = do
			{
			let {len = fromIntegral (length list);};
			array <- tlNewArray (Type::Type base) len;
			tlSetArrayRegion array 0 list;
			return array;
			};
		tlFromRef array = do
			{
			len <- tlGetArrayLength array;
			tlGetArrayRegion array 0 len;
			};
		};

	instance (IsJVMMonad m,IsJVMValue base) => MonadIsA m (ArrayRef base) [base] where
		{
		getConvert list = do
			{
			let {len = fromIntegral (length list);};
			array <- newArray (Type::Type base) len;
			setArrayRegion array 0 list;
			return array;
			};
		};

	instance (IsJVMMonad m,IsJVMValue base) => MonadIsA m [base] (ArrayRef base) where
		{
		getConvert array = do
			{
			len <- getArrayLength array;
			getArrayRegion array 0 len;
			};
		};

	instance (IsJVMValue base) => IsJVMArgumentList [base] where
		{
		getListType t = [getValueType t];
		tlAppendListToValueList = tlAppendArgumentToValueList;
		tlExtractValues vl i = do
			{
			ref <- callIOWithEnv (vmGetNthValueAsRef vl i);
			val <- tlFromRef ref;
			return (val,1);
			};
		};
	
	toArray :: (IsJVMMonad m,IsJVMValue a) => [a] -> m (ArrayRef a);
	toArray list = do
		{
		array <- tlToRef list;
		return (MkTLRef array);
		};
	
	fromArray :: (IsJVMMonad m,IsJVMValue a) => ArrayRef a -> m [a];
	fromArray (MkTLRef array) = tlFromRef array;
	}
