-- This is written in Haskell.
{--
JVM-Bridge -- bridge from FP languages and others to the Java VM
Copyright (C) 2001 Ashley Yakeley <ashley@semantic.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--}

module TypedMethod
	(
	getFunctionType,
	getStaticMethodbyType,getMethodByType,getNonvirtualMethodByType,
	getStaticMethod,getMethod,getNonvirtualMethod,
	callStaticMethod,callMethod,callNonvirtualMethod
	) where
	{
	import TypedClass;
	import TypedObject;
	import TypedReference;
	import TypedArgumentList;
	import TypedReturnable;
	import TypedMonad;
	import VMLayer;
	import JavaLayer;
	import BasicLayer;
	import MarshalArray;

	tlGetStaticMethodID :: (IsJVMMonad m) => VMRef -> MemberName -> FunctionType -> m VMStaticMethodID;
	tlGetStaticMethodID jClass name ftype = callIOWithEnv (vmGetStaticMethodID jClass name ftype);

	tlGetMethodID :: (IsJVMMonad m) => VMRef -> MemberName -> FunctionType -> m VMMethodID;
	tlGetMethodID jClass name ftype = callIOWithEnv (vmGetMethodID jClass name ftype);

	getFunctionType :: (IsJVMArgumentList args,IsJVMReturnable ret) => Type args -> Type ret -> FunctionType;
	getFunctionType a r = MkFunctionType (getListType a) (getReturnableType r);

	getStaticMethodbyType :: (IsJVMMonad m,IsJVMMonad mm,IsJVMArgumentList args,IsJVMReturnable ret) =>
		JClass -> MemberName -> Type args -> Type ret -> m (args -> mm ret);
	getStaticMethodbyType (MkTLRef jClass) name argType retType = do
		{
		methodID <- tlGetStaticMethodID jClass name (getFunctionType argType retType);
		return (\args -> do
			{
			vList <- callIO vmCreateValueList;
			tlAppendListToValueList vList args;
			tlCallStaticMethodReturning jClass methodID vList;
			});
		};

	getMethodByType :: (IsJVMMonad m,IsJVMMonad mm,IsJavaClassMarker c,IsJVMArgumentList args,IsJVMReturnable ret) =>
		JClass -> MemberName -> Type args -> Type ret -> m (ObjectRef c -> args -> mm ret);
	getMethodByType (MkTLRef jClass) name argType retType = do
		{
		methodID <- tlGetMethodID jClass name (getFunctionType argType retType);
		return (\obj -> \args -> do
			{
			vList <- callIO vmCreateValueList;
			tlAppendListToValueList vList args;
			rawObj <- tlToRef obj;
			tlCallMethodReturning rawObj methodID vList;
			});
		};

	getNonvirtualMethodByType :: (IsJVMMonad m,IsJVMMonad mm,IsJavaClassMarker c,IsJVMArgumentList args,IsJVMReturnable ret) =>
		JClass -> MemberName -> Type args -> Type ret -> m (ObjectRef c -> args -> mm ret);
	getNonvirtualMethodByType (MkTLRef jClass) name argType retType = do
		{
		methodID <- tlGetMethodID jClass name (getFunctionType argType retType);
		return (\obj args -> do
			{
			vList <- callIO vmCreateValueList;
			tlAppendListToValueList vList args;
			rawObj <- tlToRef obj;
			tlCallNonvirtualMethodReturning rawObj jClass methodID vList;
			});
		};

	getStaticMethod :: (IsJVMMonad m,IsJVMMonad mm,IsJVMArgumentList args,IsJVMReturnable ret) =>
		JClass -> MemberName -> m (args -> mm ret);
	getStaticMethod cls name =
		getStaticMethodbyType cls name Type Type;

	getMethod :: (IsJVMMonad m,IsJVMMonad mm,IsJavaClassMarker c,IsJVMArgumentList args,IsJVMReturnable ret) =>
		JClass -> MemberName -> m (ObjectRef c -> args -> mm ret);
	getMethod cls name =
		getMethodByType cls name Type Type;

	getNonvirtualMethod :: (IsJVMMonad m,IsJVMMonad mm,IsJavaClassMarker c,IsJVMArgumentList args,IsJVMReturnable ret) =>
		JClass -> MemberName -> m (ObjectRef c -> args -> mm ret);
	getNonvirtualMethod cls name =
		getNonvirtualMethodByType cls name Type Type;
	
	callStaticMethod :: (IsJVMMonad m,IsJVMArgumentList args,IsJVMReturnable ret) =>
		JClass -> MemberName -> args -> m ret;
	callStaticMethod cls name args = do
		{
		method <- getStaticMethod cls name;
		method args;
		};

	callMethod :: (IsJVMMonad m,IsJavaClassMarker c,IsJVMArgumentList args,IsJVMReturnable ret) =>
		ObjectRef c -> MemberName -> args -> m ret;
	callMethod obj name args = do
		{
		cls <- getClass obj;
		method <- getMethod cls name;
		method obj args;
		};

	callNonvirtualMethod :: (IsJVMMonad m,IsJavaClassMarker c,IsJVMArgumentList args,IsJVMReturnable ret) =>
		JClass -> ObjectRef c -> MemberName -> args -> m ret;
	callNonvirtualMethod cls obj name args = do
		{
		method <- getNonvirtualMethod cls name;
		method obj args;
		};
	}
