-- This is written in Haskell.
{--
JVM-Bridge -- bridge from FP languages and others to the Java VM
Copyright (C) 2001 Ashley Yakeley <ashley@semantic.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--}

module TypedNewObject
	(
	UnconstructedObject(),
	allocObject,getConstructor,getMakeNewObject,newObject
	) where
	{
	import TypedMethod;
	import TypedClass;
	import TypedObject;
	import TypedReference;
	import TypedArgumentList;
	import TypedReturnable;
	import TypedMonad;
	import VMLayer;
	import JavaLayer;
	import BasicLayer;
	import MarshalArray;

	newtype UnconstructedObject c = MkUnconstructedObject VMRef;
	
	allocObject :: (IsJVMMonad m,IsJavaClassMarker c) => JClass -> m (UnconstructedObject c);
	allocObject (MkTLRef cls) = do
		{
		ref <- callIOWithEnv (vmAllocObject cls);
		return (MkUnconstructedObject ref);
		};

	getConstructor :: (IsJVMMonad m,IsJVMMonad mm,IsJavaClassMarker c,IsJVMArgumentList args) =>
		JClass -> m (UnconstructedObject c -> args -> mm (ObjectRef c));
	getConstructor cls = do
		{
		constructor <- getNonvirtualMethod cls constructorName;
		return (\(MkUnconstructedObject objRef) args -> do
			{
			obj <- tlFromRef objRef;
			() <- constructor obj args;
			return obj;
			});
		};

	getMakeNewObject :: (IsJVMMonad m,IsJVMMonad mm,IsJavaClassMarker c,IsJVMArgumentList args) =>
		JClass -> m (args -> mm (ObjectRef c));
	getMakeNewObject cls = do
		{
		constructor <- getConstructor cls;
		return (\args -> do
			{
			rawObj <- allocObject cls;
			constructor rawObj args;			
			});
		};
	
	newObject :: (IsJVMMonad m,IsJavaClassMarker c,IsJVMArgumentList args) =>
		JClass -> args -> m (ObjectRef c);
	newObject cls args = do
		{
		makeNewObject <- getMakeNewObject cls;
		makeNewObject args;
		};
	}
