-- This is written in Haskell.
{--
JVM-Bridge -- bridge from FP languages and others to the Java VM
Copyright (C) 2001 Ashley Yakeley <ashley@semantic.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--}

module TypedPrimitive
	(
	) where
	{
	import TypedReturnable;
	import TypedArgumentList;
	import TypedValue;
	import TypedClass;
	import TypedMonad;
	import VMLayer;
	import JavaLayer;
	import BasicLayer;
	import MarshalArray;
	
	instance IsJVMReturnable Jboolean where
		{
		getReturnableType t = MkValueType (getValueType t);
		tlCallStaticMethodReturning classObj method vlist =
			callIOWithEnv (vmCallStaticBooleanMethod classObj method vlist);
		tlCallMethodReturning obj method vlist =
			callIOWithEnv (vmCallBooleanMethod obj method vlist);
		tlCallNonvirtualMethodReturning obj classObj method vlist =
			callIOWithEnv (vmCallNonvirtualBooleanMethod obj classObj method vlist);
		tlMakeCallback foo = do
			{
			runWithEnv <- getRunWithEnv;
			callIO (vmMakeBooleanCallback (\vl -> runWithEnv ?jvmenv (foo vl)));
			};
		};
	
	instance IsJVMReturnable Jbyte where
		{
		getReturnableType t = MkValueType (getValueType t);
		tlCallStaticMethodReturning classObj method vlist =
			callIOWithEnv (vmCallStaticByteMethod classObj method vlist);
		tlCallMethodReturning obj method vlist =
			callIOWithEnv (vmCallByteMethod obj method vlist);
		tlCallNonvirtualMethodReturning obj classObj method vlist =
			callIOWithEnv (vmCallNonvirtualByteMethod obj classObj method vlist);
		tlMakeCallback foo = do
			{
			runWithEnv <- getRunWithEnv;
			callIO (vmMakeByteCallback (\vl -> runWithEnv ?jvmenv (foo vl)));
			};
		};
	
	instance IsJVMReturnable Jchar where
		{
		getReturnableType t = MkValueType (getValueType t);
		tlCallStaticMethodReturning classObj method vlist =
			callIOWithEnv (vmCallStaticCharMethod classObj method vlist);
		tlCallMethodReturning obj method vlist =
			callIOWithEnv (vmCallCharMethod obj method vlist);
		tlCallNonvirtualMethodReturning obj classObj method vlist =
			callIOWithEnv (vmCallNonvirtualCharMethod obj classObj method vlist);
		tlMakeCallback foo = do
			{
			runWithEnv <- getRunWithEnv;
			callIO (vmMakeCharCallback (\vl -> runWithEnv ?jvmenv (foo vl)));
			};
		};
	
	instance IsJVMReturnable Jshort where
		{
		getReturnableType t = MkValueType (getValueType t);
		tlCallStaticMethodReturning classObj method vlist =
			callIOWithEnv (vmCallStaticShortMethod classObj method vlist);
		tlCallMethodReturning obj method vlist =
			callIOWithEnv (vmCallShortMethod obj method vlist);
		tlCallNonvirtualMethodReturning obj classObj method vlist =
			callIOWithEnv (vmCallNonvirtualShortMethod obj classObj method vlist);
		tlMakeCallback foo = do
			{
			runWithEnv <- getRunWithEnv;
			callIO (vmMakeShortCallback (\vl -> runWithEnv ?jvmenv (foo vl)));
			};
		};
	
	instance IsJVMReturnable Jint where
		{
		getReturnableType t = MkValueType (getValueType t);
		tlCallStaticMethodReturning classObj method vlist =
			callIOWithEnv (vmCallStaticIntMethod classObj method vlist);
		tlCallMethodReturning obj method vlist =
			callIOWithEnv (vmCallIntMethod obj method vlist);
		tlCallNonvirtualMethodReturning obj classObj method vlist =
			callIOWithEnv (vmCallNonvirtualIntMethod obj classObj method vlist);
		tlMakeCallback foo = do
			{
			runWithEnv <- getRunWithEnv;
			callIO (vmMakeIntCallback (\vl -> runWithEnv ?jvmenv (foo vl)));
			};
		};
	
	instance IsJVMReturnable Jlong where
		{
		getReturnableType t = MkValueType (getValueType t);
		tlCallStaticMethodReturning classObj method vlist =
			callIOWithEnv (vmCallStaticLongMethod classObj method vlist);
		tlCallMethodReturning obj method vlist =
			callIOWithEnv (vmCallLongMethod obj method vlist);
		tlCallNonvirtualMethodReturning obj classObj method vlist =
			callIOWithEnv (vmCallNonvirtualLongMethod obj classObj method vlist);
		tlMakeCallback foo = do
			{
			runWithEnv <- getRunWithEnv;
			callIO (vmMakeLongCallback (\vl -> runWithEnv ?jvmenv (foo vl)));
			};
		};
	
	instance IsJVMReturnable Jfloat where
		{
		getReturnableType t = MkValueType (getValueType t);
		tlCallStaticMethodReturning classObj method vlist =
			callIOWithEnv (vmCallStaticFloatMethod classObj method vlist);
		tlCallMethodReturning obj method vlist =
			callIOWithEnv (vmCallFloatMethod obj method vlist);
		tlCallNonvirtualMethodReturning obj classObj method vlist =
			callIOWithEnv (vmCallNonvirtualFloatMethod obj classObj method vlist);
		tlMakeCallback foo = do
			{
			runWithEnv <- getRunWithEnv;
			callIO (vmMakeFloatCallback (\vl -> runWithEnv ?jvmenv (foo vl)));
			};
		};
	
	instance IsJVMReturnable Jdouble where
		{
		getReturnableType t = MkValueType (getValueType t);
		tlCallStaticMethodReturning classObj method vlist =
			callIOWithEnv (vmCallStaticDoubleMethod classObj method vlist);
		tlCallMethodReturning obj method vlist =
			callIOWithEnv (vmCallDoubleMethod obj method vlist);
		tlCallNonvirtualMethodReturning obj classObj method vlist =
			callIOWithEnv (vmCallNonvirtualDoubleMethod obj classObj method vlist);
		tlMakeCallback foo = do
			{
			runWithEnv <- getRunWithEnv;
			callIO (vmMakeDoubleCallback (\vl -> runWithEnv ?jvmenv (foo vl)));
			};
		};
	
	getElementFromRegion :: (IsJVMMonad m) => (VMRef -> Jint -> Jint -> m [t])
		-> VMRef -> Jint -> m t;
	getElementFromRegion getr array i = do
		{
		region <- getr array i 1;
		case region of
			{
			(e:_)	-> return e;
			[]	-> fail "";
			};
		};
	
	setElementToRegion :: (IsJVMMonad m) => (VMRef -> Jint -> [t] -> m ())
		-> VMRef -> Jint -> t -> m ();
	setElementToRegion setr array i elem = setr array i [elem];
	
	instance IsJVMValue Jboolean where
		{
		getValueType Type = MkBooleanType;
		tlAppendArgumentToValueList vlist obj = callIO (vmAddBooleanToValueList vlist obj);
		tlGetNthValueAs vList i = callIO (vmGetNthValueAsBoolean vList i);

		tlGetStaticField cls id = callIOWithEnv (vmGetStaticBooleanField cls id);
		tlSetStaticField cls id val = callIOWithEnv (vmSetStaticBooleanField cls id val);
		tlGetField obj id = callIOWithEnv (vmGetBooleanField obj id);
		tlSetField obj id val = callIOWithEnv (vmSetBooleanField obj id val);

		tlGetArrayElement = getElementFromRegion tlGetArrayRegion;
		tlSetArrayElement = setElementToRegion tlSetArrayRegion;

		tlNewArray Type size =
			callIOWithEnv (vmNewBooleanArray size);
		tlGetArrayRegion array start len = callIOWithEnv (vmGetBooleanArrayRegion array start len);
		tlSetArrayRegion array i list = callIOWithEnv (vmSetBooleanArrayRegion array i list);
		};
	
	instance IsJVMValue Jbyte where
		{
		getValueType Type = MkByteType;
		tlAppendArgumentToValueList vlist obj = callIO (vmAddByteToValueList vlist obj);
		tlGetNthValueAs vList i = callIO (vmGetNthValueAsByte vList i);

		tlGetStaticField cls id = callIOWithEnv (vmGetStaticByteField cls id);
		tlSetStaticField cls id val = callIOWithEnv (vmSetStaticByteField cls id val);
		tlGetField obj id = callIOWithEnv (vmGetByteField obj id);
		tlSetField obj id val = callIOWithEnv (vmSetByteField obj id val);

		tlGetArrayElement = getElementFromRegion tlGetArrayRegion;
		tlSetArrayElement = setElementToRegion tlSetArrayRegion;

		tlNewArray Type size =
			callIOWithEnv (vmNewByteArray size);
		tlGetArrayRegion array start len = callIOWithEnv (vmGetByteArrayRegion array start len);
		tlSetArrayRegion array i list = callIOWithEnv (vmSetByteArrayRegion array i list);
		};
	
	instance IsJVMValue Jchar where
		{
		getValueType Type = MkCharType;
		tlAppendArgumentToValueList vlist obj = callIO (vmAddCharToValueList vlist obj);
		tlGetNthValueAs vList i = callIO (vmGetNthValueAsChar vList i);

		tlGetStaticField cls id = callIOWithEnv (vmGetStaticCharField cls id);
		tlSetStaticField cls id val = callIOWithEnv (vmSetStaticCharField cls id val);
		tlGetField obj id = callIOWithEnv (vmGetCharField obj id);
		tlSetField obj id val = callIOWithEnv (vmSetCharField obj id val);

		tlGetArrayElement = getElementFromRegion tlGetArrayRegion;
		tlSetArrayElement = setElementToRegion tlSetArrayRegion;

		tlNewArray Type size =
			callIOWithEnv (vmNewCharArray size);
		tlGetArrayRegion array start len = callIOWithEnv (vmGetCharArrayRegion array start len);
		tlSetArrayRegion array i list = callIOWithEnv (vmSetCharArrayRegion array i list);
		};
	
	instance IsJVMValue Jshort where
		{
		getValueType Type = MkShortType;
		tlAppendArgumentToValueList vlist obj = callIO (vmAddShortToValueList vlist obj);
		tlGetNthValueAs vList i = callIO (vmGetNthValueAsShort vList i);

		tlGetStaticField cls id = callIOWithEnv (vmGetStaticShortField cls id);
		tlSetStaticField cls id val = callIOWithEnv (vmSetStaticShortField cls id val);
		tlGetField obj id = callIOWithEnv (vmGetShortField obj id);
		tlSetField obj id val = callIOWithEnv (vmSetShortField obj id val);

		tlGetArrayElement = getElementFromRegion tlGetArrayRegion;
		tlSetArrayElement = setElementToRegion tlSetArrayRegion;

		tlNewArray Type size =
			callIOWithEnv (vmNewShortArray size);
		tlGetArrayRegion array start len = callIOWithEnv (vmGetShortArrayRegion array start len);
		tlSetArrayRegion array i list = callIOWithEnv (vmSetShortArrayRegion array i list);
		};
	
	instance IsJVMValue Jint where
		{
		getValueType Type = MkIntType;
		tlAppendArgumentToValueList vlist obj = callIO (vmAddIntToValueList vlist obj);
		tlGetNthValueAs vList i = callIO (vmGetNthValueAsInt vList i);

		tlGetStaticField cls id = callIOWithEnv (vmGetStaticIntField cls id);
		tlSetStaticField cls id val = callIOWithEnv (vmSetStaticIntField cls id val);
		tlGetField obj id = callIOWithEnv (vmGetIntField obj id);
		tlSetField obj id val = callIOWithEnv (vmSetIntField obj id val);

		tlGetArrayElement = getElementFromRegion tlGetArrayRegion;
		tlSetArrayElement = setElementToRegion tlSetArrayRegion;

		tlNewArray Type size =
			callIOWithEnv (vmNewIntArray size);
		tlGetArrayRegion array start len = callIOWithEnv (vmGetIntArrayRegion array start len);
		tlSetArrayRegion array i list = callIOWithEnv (vmSetIntArrayRegion array i list);
		};
	
	instance IsJVMValue Jlong where
		{
		getValueType Type = MkLongType;
		tlAppendArgumentToValueList vlist obj = callIO (vmAddLongToValueList vlist obj);
		tlGetNthValueAs vList i = callIO (vmGetNthValueAsLong vList i);

		tlGetStaticField cls id = callIOWithEnv (vmGetStaticLongField cls id);
		tlSetStaticField cls id val = callIOWithEnv (vmSetStaticLongField cls id val);
		tlGetField obj id = callIOWithEnv (vmGetLongField obj id);
		tlSetField obj id val = callIOWithEnv (vmSetLongField obj id val);

		tlGetArrayElement = getElementFromRegion tlGetArrayRegion;
		tlSetArrayElement = setElementToRegion tlSetArrayRegion;

		tlNewArray Type size =
			callIOWithEnv (vmNewLongArray size);
		tlGetArrayRegion array start len = callIOWithEnv (vmGetLongArrayRegion array start len);
		tlSetArrayRegion array i list = callIOWithEnv (vmSetLongArrayRegion array i list);
		};
	
	instance IsJVMValue Jfloat where
		{
		getValueType Type = MkFloatType;
		tlAppendArgumentToValueList vlist obj = callIO (vmAddFloatToValueList vlist obj);
		tlGetNthValueAs vList i = callIO (vmGetNthValueAsFloat vList i);

		tlGetStaticField cls id = callIOWithEnv (vmGetStaticFloatField cls id);
		tlSetStaticField cls id val = callIOWithEnv (vmSetStaticFloatField cls id val);
		tlGetField obj id = callIOWithEnv (vmGetFloatField obj id);
		tlSetField obj id val = callIOWithEnv (vmSetFloatField obj id val);

		tlGetArrayElement = getElementFromRegion tlGetArrayRegion;
		tlSetArrayElement = setElementToRegion tlSetArrayRegion;

		tlNewArray Type size =
			callIOWithEnv (vmNewFloatArray size);
		tlGetArrayRegion array start len = callIOWithEnv (vmGetFloatArrayRegion array start len);
		tlSetArrayRegion array i list = callIOWithEnv (vmSetFloatArrayRegion array i list);
		};
	
	instance IsJVMValue Jdouble where
		{
		getValueType Type = MkDoubleType;
		tlAppendArgumentToValueList vlist obj = callIO (vmAddDoubleToValueList vlist obj);
		tlGetNthValueAs vList i = callIO (vmGetNthValueAsDouble vList i);

		tlGetStaticField cls id = callIOWithEnv (vmGetStaticDoubleField cls id);
		tlSetStaticField cls id val = callIOWithEnv (vmSetStaticDoubleField cls id val);
		tlGetField obj id = callIOWithEnv (vmGetDoubleField obj id);
		tlSetField obj id val = callIOWithEnv (vmSetDoubleField obj id val);

		tlGetArrayElement = getElementFromRegion tlGetArrayRegion;
		tlSetArrayElement = setElementToRegion tlSetArrayRegion;

		tlNewArray Type size =
			callIOWithEnv (vmNewDoubleArray size);
		tlGetArrayRegion array start len = callIOWithEnv (vmGetDoubleArrayRegion array start len);
		tlSetArrayRegion array i list = callIOWithEnv (vmSetDoubleArrayRegion array i list);
		};

{-- GHC isn't smart enough to deal with this without complaining of overlapping instances.
	instance (IsJVMValue t) => IsJVMArgumentList t where
		{
		getListType t = [getValueType t];
		tlAppendListToValueList = tlAppendArgumentToValueList;
		};
--}	

	instance IsJVMArgumentList Jboolean where
		{
		getListType t = [getValueType t];
		tlAppendListToValueList = tlAppendArgumentToValueList;
		tlExtractValues vl i = do
			{
			val <- callIO (vmGetNthValueAsBoolean vl i);
			return (val,1);
			};
		};
	
	instance IsJVMArgumentList Jbyte where
		{
		getListType t = [getValueType t];
		tlAppendListToValueList = tlAppendArgumentToValueList;
		tlExtractValues vl i = do
			{
			val <- callIO (vmGetNthValueAsByte vl i);
			return (val,1);
			};
		};
	
	instance IsJVMArgumentList Jchar where
		{
		getListType t = [getValueType t];
		tlAppendListToValueList = tlAppendArgumentToValueList;
		tlExtractValues vl i = do
			{
			val <- callIO (vmGetNthValueAsChar vl i);
			return (val,1);
			};
		};
	
	instance IsJVMArgumentList Jshort where
		{
		getListType t = [getValueType t];
		tlAppendListToValueList = tlAppendArgumentToValueList;
		tlExtractValues vl i = do
			{
			val <- callIO (vmGetNthValueAsShort vl i);
			return (val,1);
			};
		};
	
	instance IsJVMArgumentList Jint where
		{
		getListType t = [getValueType t];
		tlAppendListToValueList = tlAppendArgumentToValueList;
		tlExtractValues vl i = do
			{
			val <- callIO (vmGetNthValueAsInt vl i);
			return (val,1);
			};
		};
	
	instance IsJVMArgumentList Jlong where
		{
		getListType t = [getValueType t];
		tlAppendListToValueList = tlAppendArgumentToValueList;
		tlExtractValues vl i = do
			{
			val <- callIO (vmGetNthValueAsLong vl i);
			return (val,1);
			};
		};
	
	instance IsJVMArgumentList Jfloat where
		{
		getListType t = [getValueType t];
		tlAppendListToValueList = tlAppendArgumentToValueList;
		tlExtractValues vl i = do
			{
			val <- callIO (vmGetNthValueAsFloat vl i);
			return (val,1);
			};
		};
	
	instance IsJVMArgumentList Jdouble where
		{
		getListType t = [getValueType t];
		tlAppendListToValueList = tlAppendArgumentToValueList;
		tlExtractValues vl i = do
			{
			val <- callIO (vmGetNthValueAsDouble vl i);
			return (val,1);
			};
		};
	}
