-- This is written in Haskell.
{--
JVM-Bridge -- bridge from FP languages and others to the Java VM
Copyright (C) 2001 Ashley Yakeley <ashley@semantic.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--}

module TypedReference
	(
	IsJVMReference(..),
	TLRef(MkTLRef),castTLRef,
	tlFindClassByName,tlFindClass,
	IsJReferenceFlavour(..)
	) where
	{
	import TypedArgumentList;
	import TypedValue;
	import TypedReturnable;
	import TypedMonad;
	import VMLayer;
	import JavaLayer;
	import BasicLayer;
	
	class (IsJVMValue t) => IsJVMReference t where
		{
		findClassName	:: Type t -> ClassName;	-- name for the purposes of findClass
		tlToRef			:: IsJVMMonad m => t -> m VMRef;
		tlFromRef		:: IsJVMMonad m => VMRef -> m t;
		};

	tlFindClassByName :: IsJVMMonad m => ClassName -> m VMRef;
	tlFindClassByName className = callIOWithEnv (vmFindClass className);

	tlFindClass :: (IsJVMMonad m,IsJVMReference t) => Type t -> m VMRef;
	tlFindClass t = tlFindClassByName (findClassName t);
	
	newtype TLRef c = MkTLRef VMRef;
	
	castTLRef :: TLRef a -> TLRef b;
	castTLRef (MkTLRef r) = (MkTLRef r);
	
	instance (IsJVMMonad m) => MonadHasNothing m (TLRef c) where
		{
		getNothing = do
			{
			nullRef <- callIO getNothing;
			return (MkTLRef nullRef);
			};

		getIsNothing (MkTLRef ref) = callIO (getIsNothing ref);
		};
	
	instance (IsJVMMonad m) => MonadEquals m (TLRef a) (TLRef b) where
		{
		getEquals (MkTLRef a) (MkTLRef b) = callIOWithEnv (vmIsSameObject a b);
		};
	
	class IsJReferenceFlavour t where
		{
		tlRefValueType		:: Type t -> ValueType;
		tlNameForFindClass	:: Type t -> ClassName;	-- name for the purposes of findClass
		};
	
	instance (IsJReferenceFlavour t) => IsJVMReturnable (TLRef t) where
		{
		getReturnableType t = MkValueType (getValueType t);
		tlCallStaticMethodReturning classRef method vlist = do
			{
			result <- callIOWithEnv (vmCallStaticRefMethod classRef method vlist);
			tlFromRef result;
			};
		tlCallMethodReturning obj method vlist = do
			{
			result <- callIOWithEnv (vmCallRefMethod obj method vlist);
			tlFromRef result;
			};
		tlCallNonvirtualMethodReturning obj classObj method vlist = do
			{
			result <- callIOWithEnv (vmCallNonvirtualRefMethod obj classObj method vlist);
			tlFromRef result;
			};
		tlMakeCallback foo = do
			{
			runWithEnv <- getRunWithEnv;
			callIOWithMap (\map -> vmMakeRefCallback (\vl -> do
				{
				ret <- runWithEnv ?jvmenv (foo vl);
				map (tlToRef ret);
				}));
			};
		};
	
	instance (IsJReferenceFlavour t) => IsJVMValue (TLRef t) where
		{
		getValueType t = tlRefValueType (tMap t) where
			{
			tMap :: Type (TLRef t) -> Type t;
			tMap Type = Type;
			};
		tlAppendArgumentToValueList vlist r = do
			{
			ref <- tlToRef r;
			callIO (vmAddRefToValueList vlist ref);
			};

		tlGetNthValueAs vList i = do
			{
			ref <- callIOWithEnv (vmGetNthValueAsRef vList i);
			tlFromRef ref;
			};

		tlGetStaticField cls id = do
			{
			ref <- callIOWithEnv (vmGetStaticRefField cls id);
			tlFromRef ref;
			};

		tlSetStaticField cls id val = do
			{
			ref <- tlToRef val;
			callIOWithEnv (vmSetStaticRefField cls id ref);
			};

		tlGetField obj id = do
			{
			ref <- callIOWithEnv (vmGetRefField obj id);
			tlFromRef ref;
			};

		tlSetField obj id val = do
			{
			ref <- tlToRef val;
			callIOWithEnv (vmSetRefField obj id ref);
			};

		tlNewArray t size = do
			{
			elementClass <- tlFindClass t;
			nullRef <- callIO getNothing;
			callIOWithEnv (vmNewRefArray size elementClass nullRef);
			};

		tlGetArrayElement array i = do
			{
			ref <- callIOWithEnv (vmGetRefArrayElement array i);
			tlFromRef ref;
			};

		tlGetArrayRegion array start len = if len <=0 then return [] else do
			{
			elem <- tlGetArrayElement array start;
			rest <- tlGetArrayRegion array (start + 1) (len - 1);
			return (elem:rest);
			};

		tlSetArrayElement array i elem = do
			{
			elemRef <- tlToRef elem;
			callIOWithEnv (vmSetRefArrayElement array i elemRef);
			};

		tlSetArrayRegion array start [] = return ();
		tlSetArrayRegion array start (x:xs) = do
			{
			tlSetArrayElement array start x;
			tlSetArrayRegion array (start+1) xs;
			};
		};
	
	instance (IsJReferenceFlavour t) => IsJVMArgumentList (TLRef t) where
		{
		getListType t = [getValueType t];
		tlAppendListToValueList = tlAppendArgumentToValueList;
		tlExtractValues vl i = do
			{
			ref <- callIOWithEnv (vmGetNthValueAsRef vl i);
			val <- tlFromRef ref;
			return (val,1);
			};
		};
		
	instance (IsJReferenceFlavour t) => IsJVMReference (TLRef t) where
		{
		findClassName t = tlNameForFindClass (tMap t) where
			{
			tMap :: Type (TLRef t) -> Type t;
			tMap Type = Type;
			};
		tlToRef (MkTLRef ref) = return ref;
		tlFromRef ref = return (MkTLRef ref);
		};
	}
