-- This is written in Haskell.
{--
JVM-Bridge -- bridge from FP languages and others to the Java VM
Copyright (C) 2001 Ashley Yakeley <ashley@semantic.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--}

module TypedReturnable
	(
	IsJVMReturnable(..),
	returnVoid,
	returnBoolean,returnByte,returnChar,
	returnShort,returnInt,returnLong,
	returnFloat,returnDouble
	) where
	{
	import TypedMonad;
	import VMLayer;
	import JavaLayer;
	import BasicLayer;

	class IsJVMReturnable t where
		{
		getReturnableType				:: Type t -> ReturnType;
		tlCallStaticMethodReturning		:: IsJVMMonad m => VMRef -> VMStaticMethodID -> VMValueList -> m t;
		tlCallMethodReturning			:: IsJVMMonad m => VMRef -> VMMethodID -> VMValueList -> m t;
		tlCallNonvirtualMethodReturning	:: IsJVMMonad m => VMRef -> VMRef -> VMMethodID -> VMValueList -> m t;
		tlMakeCallback					:: IsJVMMonad m => (VMValueList -> m t) -> m OpaqueAddress;
		};

	instance IsJVMReturnable Jvoid where
		{
		getReturnableType Type = MkVoidType;
		tlCallStaticMethodReturning classRef method vlist =
		 callIOWithEnv (vmCallStaticVoidMethod classRef method vlist);
		tlCallMethodReturning obj method vlist =
		 callIOWithEnv (vmCallVoidMethod obj method vlist);
		tlCallNonvirtualMethodReturning obj classObj method vlist =
		 callIOWithEnv (vmCallNonvirtualVoidMethod obj classObj method vlist);
		tlMakeCallback foo = do
			{
			runWithEnv <- getRunWithEnv;
			callIO (vmMakeVoidCallback (\vl -> runWithEnv ?jvmenv (foo vl)));
			};
		};

	returnVoid		= Type::Type Jvoid;
	returnBoolean	= Type::Type Jboolean;
	returnByte		= Type::Type Jbyte;
	returnChar		= Type::Type Jchar;
	returnShort		= Type::Type Jshort;
	returnInt		= Type::Type Jint;
	returnLong		= Type::Type Jlong;
	returnFloat		= Type::Type Jfloat;
	returnDouble	= Type::Type Jdouble;
	}
