-- This is written in Haskell.
{--
JVM-Bridge -- bridge from FP languages and others to the Java VM
Copyright (C) 2001 Ashley Yakeley <ashley@semantic.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--}

module TypedTuple
	(
	) where
	{	
	import TypedArgumentList;
	import TypedValue;
	import TypedReturnable;
	import BasicLayer;
	
	-- 0
	instance IsJVMArgumentList () where
		{
		getListType Type = [];
		tlAppendListToValueList _ () = return ();
		tlExtractValues vlist ii = return ((),0);
		};

	-- 2
	instance (IsJVMValue a,IsJVMValue b) =>
		IsJVMArgumentList (a,b) where
		{
		getListType Type = 
			[
			getValueType (Type::Type a),
			getValueType (Type::Type b)
			];
		tlAppendListToValueList vlist (a,b) = do
			{
			tlAppendArgumentToValueList vlist a;
			tlAppendArgumentToValueList vlist b;
			};
		tlExtractValues vlist ii = do
			{
			a <- tlGetNthValueAs vlist ii;
			b <- tlGetNthValueAs vlist (ii+1);
			return ((a,b),2);
			};
		};
	
	-- 3
	instance (IsJVMValue a,IsJVMValue b,IsJVMValue c) =>
		IsJVMArgumentList (a,b,c) where
		{
		getListType Type =
			[
			getValueType (Type::Type a),
			getValueType (Type::Type b),
			getValueType (Type::Type c)
			];
		tlAppendListToValueList vlist (a,b,c) = do
			{
			tlAppendArgumentToValueList vlist a;
			tlAppendArgumentToValueList vlist b;
			tlAppendArgumentToValueList vlist c;
			};
		tlExtractValues vlist ii = do
			{
			a <- tlGetNthValueAs vlist ii;
			b <- tlGetNthValueAs vlist (ii+1);
			c <- tlGetNthValueAs vlist (ii+2);
			return ((a,b,c),3);
			};
		};
	
	-- 4
	instance (IsJVMValue a,IsJVMValue b,IsJVMValue c,IsJVMValue d) =>
		IsJVMArgumentList (a,b,c,d) where
		{
		getListType Type =
			[
			getValueType (Type::Type a),
			getValueType (Type::Type b),
			getValueType (Type::Type c),
			getValueType (Type::Type d)
			];
		tlAppendListToValueList vlist (a,b,c,d) = do
			{
			tlAppendArgumentToValueList vlist a;
			tlAppendArgumentToValueList vlist b;
			tlAppendArgumentToValueList vlist c;
			tlAppendArgumentToValueList vlist d;
			};
		tlExtractValues vlist ii = do
			{
			a <- tlGetNthValueAs vlist ii;
			b <- tlGetNthValueAs vlist (ii+1);
			c <- tlGetNthValueAs vlist (ii+2);
			d <- tlGetNthValueAs vlist (ii+3);
			return ((a,b,c,d),4);
			};
		};
	
	-- 5
	instance (IsJVMValue a,IsJVMValue b,IsJVMValue c,IsJVMValue d,IsJVMValue e) =>
		IsJVMArgumentList (a,b,c,d,e) where
		{
		getListType Type =
			[
			getValueType (Type::Type a),
			getValueType (Type::Type b),
			getValueType (Type::Type c),
			getValueType (Type::Type d),
			getValueType (Type::Type e)
			];
		tlAppendListToValueList vlist (a,b,c,d,e) = do
			{
			tlAppendArgumentToValueList vlist a;
			tlAppendArgumentToValueList vlist b;
			tlAppendArgumentToValueList vlist c;
			tlAppendArgumentToValueList vlist d;
			tlAppendArgumentToValueList vlist e;
			};
		tlExtractValues vlist ii = do
			{
			a <- tlGetNthValueAs vlist ii;
			b <- tlGetNthValueAs vlist (ii+1);
			c <- tlGetNthValueAs vlist (ii+2);
			d <- tlGetNthValueAs vlist (ii+3);
			e <- tlGetNthValueAs vlist (ii+4);
			return ((a,b,c,d,e),5);
			};
		};
	
	-- 6
	instance (IsJVMValue a,IsJVMValue b,IsJVMValue c,IsJVMValue d,IsJVMValue e,IsJVMValue f) =>
		IsJVMArgumentList (a,b,c,d,e,f) where
		{
		getListType Type =
			[
			getValueType (Type::Type a),
			getValueType (Type::Type b),
			getValueType (Type::Type c),
			getValueType (Type::Type d),
			getValueType (Type::Type e),
			getValueType (Type::Type f)
			];
		tlAppendListToValueList vlist (a,b,c,d,e,f) = do
			{
			tlAppendArgumentToValueList vlist a;
			tlAppendArgumentToValueList vlist b;
			tlAppendArgumentToValueList vlist c;
			tlAppendArgumentToValueList vlist d;
			tlAppendArgumentToValueList vlist e;
			tlAppendArgumentToValueList vlist f;
			};
		tlExtractValues vlist ii = do
			{
			a <- tlGetNthValueAs vlist ii;
			b <- tlGetNthValueAs vlist (ii+1);
			c <- tlGetNthValueAs vlist (ii+2);
			d <- tlGetNthValueAs vlist (ii+3);
			e <- tlGetNthValueAs vlist (ii+4);
			f <- tlGetNthValueAs vlist (ii+5);
			return ((a,b,c,d,e,f),6);
			};
		};
	
	-- 7
	instance (
		IsJVMValue a,
		IsJVMValue b,
		IsJVMValue c,
		IsJVMValue d,
		IsJVMValue e,
		IsJVMValue f,
		IsJVMValue g
		) =>
		IsJVMArgumentList (a,b,c,d,e,f,g) where
		{
		getListType Type =
			[
			getValueType (Type::Type a),
			getValueType (Type::Type b),
			getValueType (Type::Type c),
			getValueType (Type::Type d),
			getValueType (Type::Type e),
			getValueType (Type::Type f),
			getValueType (Type::Type g)
			];
		tlAppendListToValueList vlist (a,b,c,d,e,f,g) = do
			{
			tlAppendArgumentToValueList vlist a;
			tlAppendArgumentToValueList vlist b;
			tlAppendArgumentToValueList vlist c;
			tlAppendArgumentToValueList vlist d;
			tlAppendArgumentToValueList vlist e;
			tlAppendArgumentToValueList vlist f;
			tlAppendArgumentToValueList vlist g;
			};
		tlExtractValues vlist ii = do
			{
			a <- tlGetNthValueAs vlist ii;
			b <- tlGetNthValueAs vlist (ii+1);
			c <- tlGetNthValueAs vlist (ii+2);
			d <- tlGetNthValueAs vlist (ii+3);
			e <- tlGetNthValueAs vlist (ii+4);
			f <- tlGetNthValueAs vlist (ii+5);
			g <- tlGetNthValueAs vlist (ii+6);
			return ((a,b,c,d,e,f,g),7);
			};
		};
	
	-- 8
	instance (
		IsJVMValue a,
		IsJVMValue b,
		IsJVMValue c,
		IsJVMValue d,
		IsJVMValue e,
		IsJVMValue f,
		IsJVMValue g,
		IsJVMValue h
		) =>
		IsJVMArgumentList (a,b,c,d,e,f,g,h) where
		{
		getListType Type =
			[
			getValueType (Type::Type a),
			getValueType (Type::Type b),
			getValueType (Type::Type c),
			getValueType (Type::Type d),
			getValueType (Type::Type e),
			getValueType (Type::Type f),
			getValueType (Type::Type g),
			getValueType (Type::Type h)
			];
		tlAppendListToValueList vlist (a,b,c,d,e,f,g,h) = do
			{
			tlAppendArgumentToValueList vlist a;
			tlAppendArgumentToValueList vlist b;
			tlAppendArgumentToValueList vlist c;
			tlAppendArgumentToValueList vlist d;
			tlAppendArgumentToValueList vlist e;
			tlAppendArgumentToValueList vlist f;
			tlAppendArgumentToValueList vlist g;
			tlAppendArgumentToValueList vlist h;
			};
		tlExtractValues vlist ii = do
			{
			a <- tlGetNthValueAs vlist ii;
			b <- tlGetNthValueAs vlist (ii+1);
			c <- tlGetNthValueAs vlist (ii+2);
			d <- tlGetNthValueAs vlist (ii+3);
			e <- tlGetNthValueAs vlist (ii+4);
			f <- tlGetNthValueAs vlist (ii+5);
			g <- tlGetNthValueAs vlist (ii+6);
			h <- tlGetNthValueAs vlist (ii+7);
			return ((a,b,c,d,e,f,g,h),8);
			};
		};
	
	-- 9
	instance (
		IsJVMValue a,
		IsJVMValue b,
		IsJVMValue c,
		IsJVMValue d,
		IsJVMValue e,
		IsJVMValue f,
		IsJVMValue g,
		IsJVMValue h,
		IsJVMValue i
		) =>
		IsJVMArgumentList (a,b,c,d,e,f,g,h,i) where
		{
		getListType Type =
			[
			getValueType (Type::Type a),
			getValueType (Type::Type b),
			getValueType (Type::Type c),
			getValueType (Type::Type d),
			getValueType (Type::Type e),
			getValueType (Type::Type f),
			getValueType (Type::Type g),
			getValueType (Type::Type h),
			getValueType (Type::Type i)
			];
		tlAppendListToValueList vlist (a,b,c,d,e,f,g,h,i) = do
			{
			tlAppendArgumentToValueList vlist a;
			tlAppendArgumentToValueList vlist b;
			tlAppendArgumentToValueList vlist c;
			tlAppendArgumentToValueList vlist d;
			tlAppendArgumentToValueList vlist e;
			tlAppendArgumentToValueList vlist f;
			tlAppendArgumentToValueList vlist g;
			tlAppendArgumentToValueList vlist h;
			tlAppendArgumentToValueList vlist i;
			};
		tlExtractValues vlist ii = do
			{
			a <- tlGetNthValueAs vlist ii;
			b <- tlGetNthValueAs vlist (ii+1);
			c <- tlGetNthValueAs vlist (ii+2);
			d <- tlGetNthValueAs vlist (ii+3);
			e <- tlGetNthValueAs vlist (ii+4);
			f <- tlGetNthValueAs vlist (ii+5);
			g <- tlGetNthValueAs vlist (ii+6);
			h <- tlGetNthValueAs vlist (ii+7);
			i <- tlGetNthValueAs vlist (ii+8);
			return ((a,b,c,d,e,f,g,h,i),9);
			};
		};
	
	-- 10
	instance (
		IsJVMValue a,
		IsJVMValue b,
		IsJVMValue c,
		IsJVMValue d,
		IsJVMValue e,
		IsJVMValue f,
		IsJVMValue g,
		IsJVMValue h,
		IsJVMValue i,
		IsJVMValue j
		) =>
		IsJVMArgumentList (a,b,c,d,e,f,g,h,i,j) where
		{
		getListType Type =
			[
			getValueType (Type::Type a),
			getValueType (Type::Type b),
			getValueType (Type::Type c),
			getValueType (Type::Type d),
			getValueType (Type::Type e),
			getValueType (Type::Type f),
			getValueType (Type::Type g),
			getValueType (Type::Type h),
			getValueType (Type::Type i),
			getValueType (Type::Type j)
			];
		tlAppendListToValueList vlist (a,b,c,d,e,f,g,h,i,j) = do
			{
			tlAppendArgumentToValueList vlist a;
			tlAppendArgumentToValueList vlist b;
			tlAppendArgumentToValueList vlist c;
			tlAppendArgumentToValueList vlist d;
			tlAppendArgumentToValueList vlist e;
			tlAppendArgumentToValueList vlist f;
			tlAppendArgumentToValueList vlist g;
			tlAppendArgumentToValueList vlist h;
			tlAppendArgumentToValueList vlist i;
			tlAppendArgumentToValueList vlist j;
			};
		tlExtractValues vlist ii = do
			{
			a <- tlGetNthValueAs vlist ii;
			b <- tlGetNthValueAs vlist (ii+1);
			c <- tlGetNthValueAs vlist (ii+2);
			d <- tlGetNthValueAs vlist (ii+3);
			e <- tlGetNthValueAs vlist (ii+4);
			f <- tlGetNthValueAs vlist (ii+5);
			g <- tlGetNthValueAs vlist (ii+6);
			h <- tlGetNthValueAs vlist (ii+7);
			i <- tlGetNthValueAs vlist (ii+8);
			j <- tlGetNthValueAs vlist (ii+9);
			return ((a,b,c,d,e,f,g,h,i,j),10);
			};
		};
	
	-- 11
	instance (
		IsJVMValue a,
		IsJVMValue b,
		IsJVMValue c,
		IsJVMValue d,
		IsJVMValue e,
		IsJVMValue f,
		IsJVMValue g,
		IsJVMValue h,
		IsJVMValue i,
		IsJVMValue j,
		IsJVMValue k
		) =>
		IsJVMArgumentList (a,b,c,d,e,f,g,h,i,j,k) where
		{
		getListType Type =
			[
			getValueType (Type::Type a),
			getValueType (Type::Type b),
			getValueType (Type::Type c),
			getValueType (Type::Type d),
			getValueType (Type::Type e),
			getValueType (Type::Type f),
			getValueType (Type::Type g),
			getValueType (Type::Type h),
			getValueType (Type::Type i),
			getValueType (Type::Type j),
			getValueType (Type::Type k)
			];
		tlAppendListToValueList vlist (a,b,c,d,e,f,g,h,i,j,k) = do
			{
			tlAppendArgumentToValueList vlist a;
			tlAppendArgumentToValueList vlist b;
			tlAppendArgumentToValueList vlist c;
			tlAppendArgumentToValueList vlist d;
			tlAppendArgumentToValueList vlist e;
			tlAppendArgumentToValueList vlist f;
			tlAppendArgumentToValueList vlist g;
			tlAppendArgumentToValueList vlist h;
			tlAppendArgumentToValueList vlist i;
			tlAppendArgumentToValueList vlist j;
			tlAppendArgumentToValueList vlist k;
			};
		tlExtractValues vlist ii = do
			{
			a <- tlGetNthValueAs vlist ii;
			b <- tlGetNthValueAs vlist (ii+1);
			c <- tlGetNthValueAs vlist (ii+2);
			d <- tlGetNthValueAs vlist (ii+3);
			e <- tlGetNthValueAs vlist (ii+4);
			f <- tlGetNthValueAs vlist (ii+5);
			g <- tlGetNthValueAs vlist (ii+6);
			h <- tlGetNthValueAs vlist (ii+7);
			i <- tlGetNthValueAs vlist (ii+8);
			j <- tlGetNthValueAs vlist (ii+9);
			k <- tlGetNthValueAs vlist (ii+10);
			return ((a,b,c,d,e,f,g,h,i,j,k),11);
			};
		};
	
	-- 12
	instance (
		IsJVMValue a,
		IsJVMValue b,
		IsJVMValue c,
		IsJVMValue d,
		IsJVMValue e,
		IsJVMValue f,
		IsJVMValue g,
		IsJVMValue h,
		IsJVMValue i,
		IsJVMValue j,
		IsJVMValue k,
		IsJVMValue l
		) =>
		IsJVMArgumentList (a,b,c,d,e,f,g,h,i,j,k,l) where
		{
		getListType Type =
			[
			getValueType (Type::Type a),
			getValueType (Type::Type b),
			getValueType (Type::Type c),
			getValueType (Type::Type d),
			getValueType (Type::Type e),
			getValueType (Type::Type f),
			getValueType (Type::Type g),
			getValueType (Type::Type h),
			getValueType (Type::Type i),
			getValueType (Type::Type j),
			getValueType (Type::Type k),
			getValueType (Type::Type l)
			];
		tlAppendListToValueList vlist (a,b,c,d,e,f,g,h,i,j,k,l) = do
			{
			tlAppendArgumentToValueList vlist a;
			tlAppendArgumentToValueList vlist b;
			tlAppendArgumentToValueList vlist c;
			tlAppendArgumentToValueList vlist d;
			tlAppendArgumentToValueList vlist e;
			tlAppendArgumentToValueList vlist f;
			tlAppendArgumentToValueList vlist g;
			tlAppendArgumentToValueList vlist h;
			tlAppendArgumentToValueList vlist i;
			tlAppendArgumentToValueList vlist j;
			tlAppendArgumentToValueList vlist k;
			tlAppendArgumentToValueList vlist l;
			};
		tlExtractValues vlist ii = do
			{
			a <- tlGetNthValueAs vlist ii;
			b <- tlGetNthValueAs vlist (ii+1);
			c <- tlGetNthValueAs vlist (ii+2);
			d <- tlGetNthValueAs vlist (ii+3);
			e <- tlGetNthValueAs vlist (ii+4);
			f <- tlGetNthValueAs vlist (ii+5);
			g <- tlGetNthValueAs vlist (ii+6);
			h <- tlGetNthValueAs vlist (ii+7);
			i <- tlGetNthValueAs vlist (ii+8);
			j <- tlGetNthValueAs vlist (ii+9);
			k <- tlGetNthValueAs vlist (ii+10);
			l <- tlGetNthValueAs vlist (ii+11);
			return ((a,b,c,d,e,f,g,h,i,j,k,l),12);
			};
		};
	
	-- 13
	instance (
		IsJVMValue a,
		IsJVMValue b,
		IsJVMValue c,
		IsJVMValue d,
		IsJVMValue e,
		IsJVMValue f,
		IsJVMValue g,
		IsJVMValue h,
		IsJVMValue i,
		IsJVMValue j,
		IsJVMValue k,
		IsJVMValue l,
		IsJVMValue m
		) =>
		IsJVMArgumentList (a,b,c,d,e,f,g,h,i,j,k,l,m) where
		{
		getListType Type =
			[
			getValueType (Type::Type a),
			getValueType (Type::Type b),
			getValueType (Type::Type c),
			getValueType (Type::Type d),
			getValueType (Type::Type e),
			getValueType (Type::Type f),
			getValueType (Type::Type g),
			getValueType (Type::Type h),
			getValueType (Type::Type i),
			getValueType (Type::Type j),
			getValueType (Type::Type k),
			getValueType (Type::Type l),
			getValueType (Type::Type m)
			];
		tlAppendListToValueList vlist (a,b,c,d,e,f,g,h,i,j,k,l,m) = do
			{
			tlAppendArgumentToValueList vlist a;
			tlAppendArgumentToValueList vlist b;
			tlAppendArgumentToValueList vlist c;
			tlAppendArgumentToValueList vlist d;
			tlAppendArgumentToValueList vlist e;
			tlAppendArgumentToValueList vlist f;
			tlAppendArgumentToValueList vlist g;
			tlAppendArgumentToValueList vlist h;
			tlAppendArgumentToValueList vlist i;
			tlAppendArgumentToValueList vlist j;
			tlAppendArgumentToValueList vlist k;
			tlAppendArgumentToValueList vlist l;
			tlAppendArgumentToValueList vlist m;
			};
		tlExtractValues vlist ii = do
			{
			a <- tlGetNthValueAs vlist ii;
			b <- tlGetNthValueAs vlist (ii+1);
			c <- tlGetNthValueAs vlist (ii+2);
			d <- tlGetNthValueAs vlist (ii+3);
			e <- tlGetNthValueAs vlist (ii+4);
			f <- tlGetNthValueAs vlist (ii+5);
			g <- tlGetNthValueAs vlist (ii+6);
			h <- tlGetNthValueAs vlist (ii+7);
			i <- tlGetNthValueAs vlist (ii+8);
			j <- tlGetNthValueAs vlist (ii+9);
			k <- tlGetNthValueAs vlist (ii+10);
			l <- tlGetNthValueAs vlist (ii+11);
			m <- tlGetNthValueAs vlist (ii+12);
			return ((a,b,c,d,e,f,g,h,i,j,k,l,m),13);
			};
		};
	
	-- 14
	instance (
		IsJVMValue a,
		IsJVMValue b,
		IsJVMValue c,
		IsJVMValue d,
		IsJVMValue e,
		IsJVMValue f,
		IsJVMValue g,
		IsJVMValue h,
		IsJVMValue i,
		IsJVMValue j,
		IsJVMValue k,
		IsJVMValue l,
		IsJVMValue m,
		IsJVMValue n
		) =>
		IsJVMArgumentList (a,b,c,d,e,f,g,h,i,j,k,l,m,n) where
		{
		getListType Type =
			[
			getValueType (Type::Type a),
			getValueType (Type::Type b),
			getValueType (Type::Type c),
			getValueType (Type::Type d),
			getValueType (Type::Type e),
			getValueType (Type::Type f),
			getValueType (Type::Type g),
			getValueType (Type::Type h),
			getValueType (Type::Type i),
			getValueType (Type::Type j),
			getValueType (Type::Type k),
			getValueType (Type::Type l),
			getValueType (Type::Type m),
			getValueType (Type::Type n)
			];
		tlAppendListToValueList vlist (a,b,c,d,e,f,g,h,i,j,k,l,m,n) = do
			{
			tlAppendArgumentToValueList vlist a;
			tlAppendArgumentToValueList vlist b;
			tlAppendArgumentToValueList vlist c;
			tlAppendArgumentToValueList vlist d;
			tlAppendArgumentToValueList vlist e;
			tlAppendArgumentToValueList vlist f;
			tlAppendArgumentToValueList vlist g;
			tlAppendArgumentToValueList vlist h;
			tlAppendArgumentToValueList vlist i;
			tlAppendArgumentToValueList vlist j;
			tlAppendArgumentToValueList vlist k;
			tlAppendArgumentToValueList vlist l;
			tlAppendArgumentToValueList vlist m;
			tlAppendArgumentToValueList vlist n;
			};
		tlExtractValues vlist ii = do
			{
			a <- tlGetNthValueAs vlist ii;
			b <- tlGetNthValueAs vlist (ii+1);
			c <- tlGetNthValueAs vlist (ii+2);
			d <- tlGetNthValueAs vlist (ii+3);
			e <- tlGetNthValueAs vlist (ii+4);
			f <- tlGetNthValueAs vlist (ii+5);
			g <- tlGetNthValueAs vlist (ii+6);
			h <- tlGetNthValueAs vlist (ii+7);
			i <- tlGetNthValueAs vlist (ii+8);
			j <- tlGetNthValueAs vlist (ii+9);
			k <- tlGetNthValueAs vlist (ii+10);
			l <- tlGetNthValueAs vlist (ii+11);
			m <- tlGetNthValueAs vlist (ii+12);
			n <- tlGetNthValueAs vlist (ii+13);
			return ((a,b,c,d,e,f,g,h,i,j,k,l,m,n),14);
			};
		};
	
	-- 15
	instance (
		IsJVMValue a,
		IsJVMValue b,
		IsJVMValue c,
		IsJVMValue d,
		IsJVMValue e,
		IsJVMValue f,
		IsJVMValue g,
		IsJVMValue h,
		IsJVMValue i,
		IsJVMValue j,
		IsJVMValue k,
		IsJVMValue l,
		IsJVMValue m,
		IsJVMValue n,
		IsJVMValue o
		) =>
		IsJVMArgumentList (a,b,c,d,e,f,g,h,i,j,k,l,m,n,o) where
		{
		getListType Type =
			[
			getValueType (Type::Type a),
			getValueType (Type::Type b),
			getValueType (Type::Type c),
			getValueType (Type::Type d),
			getValueType (Type::Type e),
			getValueType (Type::Type f),
			getValueType (Type::Type g),
			getValueType (Type::Type h),
			getValueType (Type::Type i),
			getValueType (Type::Type j),
			getValueType (Type::Type k),
			getValueType (Type::Type l),
			getValueType (Type::Type m),
			getValueType (Type::Type n),
			getValueType (Type::Type o)
			];
		tlAppendListToValueList vlist (a,b,c,d,e,f,g,h,i,j,k,l,m,n,o) = do
			{
			tlAppendArgumentToValueList vlist a;
			tlAppendArgumentToValueList vlist b;
			tlAppendArgumentToValueList vlist c;
			tlAppendArgumentToValueList vlist d;
			tlAppendArgumentToValueList vlist e;
			tlAppendArgumentToValueList vlist f;
			tlAppendArgumentToValueList vlist g;
			tlAppendArgumentToValueList vlist h;
			tlAppendArgumentToValueList vlist i;
			tlAppendArgumentToValueList vlist j;
			tlAppendArgumentToValueList vlist k;
			tlAppendArgumentToValueList vlist l;
			tlAppendArgumentToValueList vlist m;
			tlAppendArgumentToValueList vlist n;
			tlAppendArgumentToValueList vlist o;
			};
		tlExtractValues vlist ii = do
			{
			a <- tlGetNthValueAs vlist ii;
			b <- tlGetNthValueAs vlist (ii+1);
			c <- tlGetNthValueAs vlist (ii+2);
			d <- tlGetNthValueAs vlist (ii+3);
			e <- tlGetNthValueAs vlist (ii+4);
			f <- tlGetNthValueAs vlist (ii+5);
			g <- tlGetNthValueAs vlist (ii+6);
			h <- tlGetNthValueAs vlist (ii+7);
			i <- tlGetNthValueAs vlist (ii+8);
			j <- tlGetNthValueAs vlist (ii+9);
			k <- tlGetNthValueAs vlist (ii+10);
			l <- tlGetNthValueAs vlist (ii+11);
			m <- tlGetNthValueAs vlist (ii+12);
			n <- tlGetNthValueAs vlist (ii+13);
			o <- tlGetNthValueAs vlist (ii+14);
			return ((a,b,c,d,e,f,g,h,i,j,k,l,m,n,o),15);
			};
		};
	
	-- 16
	instance (
		IsJVMValue a,
		IsJVMValue b,
		IsJVMValue c,
		IsJVMValue d,
		IsJVMValue e,
		IsJVMValue f,
		IsJVMValue g,
		IsJVMValue h,
		IsJVMValue i,
		IsJVMValue j,
		IsJVMValue k,
		IsJVMValue l,
		IsJVMValue m,
		IsJVMValue n,
		IsJVMValue o,
		IsJVMValue p
		) =>
		IsJVMArgumentList (a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p) where
		{
		getListType Type =
			[
			getValueType (Type::Type a),
			getValueType (Type::Type b),
			getValueType (Type::Type c),
			getValueType (Type::Type d),
			getValueType (Type::Type e),
			getValueType (Type::Type f),
			getValueType (Type::Type g),
			getValueType (Type::Type h),
			getValueType (Type::Type i),
			getValueType (Type::Type j),
			getValueType (Type::Type k),
			getValueType (Type::Type l),
			getValueType (Type::Type m),
			getValueType (Type::Type n),
			getValueType (Type::Type o),
			getValueType (Type::Type p)
			];
		tlAppendListToValueList vlist (a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p) = do
			{
			tlAppendArgumentToValueList vlist a;
			tlAppendArgumentToValueList vlist b;
			tlAppendArgumentToValueList vlist c;
			tlAppendArgumentToValueList vlist d;
			tlAppendArgumentToValueList vlist e;
			tlAppendArgumentToValueList vlist f;
			tlAppendArgumentToValueList vlist g;
			tlAppendArgumentToValueList vlist h;
			tlAppendArgumentToValueList vlist i;
			tlAppendArgumentToValueList vlist j;
			tlAppendArgumentToValueList vlist k;
			tlAppendArgumentToValueList vlist l;
			tlAppendArgumentToValueList vlist m;
			tlAppendArgumentToValueList vlist n;
			tlAppendArgumentToValueList vlist o;
			tlAppendArgumentToValueList vlist p;
			};
		tlExtractValues vlist ii = do
			{
			a <- tlGetNthValueAs vlist ii;
			b <- tlGetNthValueAs vlist (ii+1);
			c <- tlGetNthValueAs vlist (ii+2);
			d <- tlGetNthValueAs vlist (ii+3);
			e <- tlGetNthValueAs vlist (ii+4);
			f <- tlGetNthValueAs vlist (ii+5);
			g <- tlGetNthValueAs vlist (ii+6);
			h <- tlGetNthValueAs vlist (ii+7);
			i <- tlGetNthValueAs vlist (ii+8);
			j <- tlGetNthValueAs vlist (ii+9);
			k <- tlGetNthValueAs vlist (ii+10);
			l <- tlGetNthValueAs vlist (ii+11);
			m <- tlGetNthValueAs vlist (ii+12);
			n <- tlGetNthValueAs vlist (ii+13);
			o <- tlGetNthValueAs vlist (ii+14);
			p <- tlGetNthValueAs vlist (ii+15);
			return ((a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p),16);
			};
		};
	}
