-- This is written in Haskell.
{--
JVM-Bridge -- bridge from FP languages and others to the Java VM
Copyright (C) 2001 Ashley Yakeley <ashley@semantic.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--}

module VMArray where
	{
	import VMRef;
	import VMTypes;
	import RawLayer;
	import JavaLayer;
	import MarshalArray;
	import Foreign;

	convertNew :: (?jvmenv :: VMEnv) =>
	 (RawEnv -> RawSize -> IO RawLocalRef) -> (Jint -> IO VMRef);
	convertNew rawF s = do
		{
		loc <- rawF ?jvmenv s;
		makeVMRefFromLocal loc;
		};

	vmNewBooleanArray	:: (?jvmenv :: VMEnv) =>
	 Jint -> IO VMRef;
	vmNewBooleanArray	= convertNew rawNewBooleanArray;
	vmNewByteArray		:: (?jvmenv :: VMEnv) =>
	 Jint -> IO VMRef;
	vmNewByteArray		= convertNew rawNewByteArray;
	vmNewCharArray		:: (?jvmenv :: VMEnv) =>
	 Jint -> IO VMRef;
	vmNewCharArray		= convertNew rawNewCharArray;
	vmNewShortArray		:: (?jvmenv :: VMEnv) =>
	 Jint -> IO VMRef;
	vmNewShortArray		= convertNew rawNewShortArray;
	vmNewIntArray		:: (?jvmenv :: VMEnv) =>
	 Jint -> IO VMRef;
	vmNewIntArray		= convertNew rawNewIntArray;
	vmNewLongArray		:: (?jvmenv :: VMEnv) =>
	 Jint -> IO VMRef;
	vmNewLongArray		= convertNew rawNewLongArray;
	vmNewFloatArray		:: (?jvmenv :: VMEnv) =>
	 Jint -> IO VMRef;
	vmNewFloatArray		= convertNew rawNewFloatArray;
	vmNewDoubleArray	:: (?jvmenv :: VMEnv) =>
	 Jint -> IO VMRef;
	vmNewDoubleArray	= convertNew rawNewDoubleArray;

	vmNewRefArray		:: (?jvmenv :: VMEnv) =>
	 Jint -> VMRef -> VMRef -> IO VMRef;
	vmNewRefArray len cls obj = do
		{
		loc <-  withVMRefRawGlobal cls (\rawCls -> 
			 withVMRefRawGlobal obj (\rawObj -> 
			 	rawNewRefArray ?jvmenv len rawCls rawObj
			 	));
		makeVMRefFromLocal loc;
		};

	vmGetArrayLength	:: (?jvmenv :: VMEnv) =>
	 VMRef -> IO Jint;
	vmGetArrayLength array = withVMRefRawGlobal array (\rawArray -> rawGetArrayLength ?jvmenv rawArray);

	convertGetRegion :: (Storable a,?jvmenv :: VMEnv) =>
	 (RawEnv -> RawGlobalRef -> RawSize -> RawSize -> IO (Ptr a)) -> (VMRef -> Jint -> Jint -> IO [a]);
	convertGetRegion rawF array start len = do
		{
		ptr <- withVMRefRawGlobal array (\rawArray -> rawF ?jvmenv rawArray start len);
		peekArray (fromIntegral len) ptr;
		};
	
	vmGetBooleanArrayRegion	:: (?jvmenv :: VMEnv) =>
	 VMRef -> Jint -> Jint -> IO [Jboolean];
	vmGetBooleanArrayRegion array start len = do
		{
		region <- convertGetRegion rawGetBooleanArrayRegion array start len;
		return (fmap fromRawBoolean region);
		};

	vmGetByteArrayRegion	:: (?jvmenv :: VMEnv) =>
	 VMRef -> Jint -> Jint -> IO [Jbyte];
	vmGetByteArrayRegion	= convertGetRegion rawGetByteArrayRegion;
	vmGetCharArrayRegion	:: (?jvmenv :: VMEnv) =>
	 VMRef -> Jint -> Jint -> IO [Jchar];
	vmGetCharArrayRegion	= convertGetRegion rawGetCharArrayRegion;
	vmGetShortArrayRegion	:: (?jvmenv :: VMEnv) =>
	 VMRef -> Jint -> Jint -> IO [Jshort];
	vmGetShortArrayRegion	= convertGetRegion rawGetShortArrayRegion;
	vmGetIntArrayRegion		:: (?jvmenv :: VMEnv) =>
	 VMRef -> Jint -> Jint -> IO [Jint];
	vmGetIntArrayRegion		= convertGetRegion rawGetIntArrayRegion;
	vmGetLongArrayRegion	:: (?jvmenv :: VMEnv) =>
	 VMRef -> Jint -> Jint -> IO [Jlong];
	vmGetLongArrayRegion	= convertGetRegion rawGetLongArrayRegion;
	vmGetFloatArrayRegion	:: (?jvmenv :: VMEnv) =>
	 VMRef -> Jint -> Jint -> IO [Jfloat];
	vmGetFloatArrayRegion	= convertGetRegion rawGetFloatArrayRegion;
	vmGetDoubleArrayRegion	:: (?jvmenv :: VMEnv) =>
	 VMRef -> Jint -> Jint -> IO [Jdouble];
	vmGetDoubleArrayRegion	= convertGetRegion rawGetDoubleArrayRegion;

	vmGetRefArrayElement	:: (?jvmenv :: VMEnv) =>
	 VMRef -> Jint -> IO VMRef;
	vmGetRefArrayElement array i = do	
		{
		elem <- withVMRefRawGlobal array (\rawArray -> rawGetRefArrayElement ?jvmenv rawArray i);
		makeVMRefFromLocal elem;
		};

	convertSetRegion :: (Storable a,?jvmenv :: VMEnv) =>
	 (RawEnv -> RawGlobalRef -> RawSize -> RawSize -> Ptr a -> IO ()) -> (VMRef -> Jint -> [a] -> IO ());
	convertSetRegion rawF array start region = withArray region (\regionPtr ->
		withVMRefRawGlobal array (\rawArray ->
			rawF ?jvmenv rawArray start (fromIntegral (length region)) regionPtr
			));

	vmSetBooleanArrayRegion	:: (?jvmenv :: VMEnv) =>
	 VMRef -> Jint -> [Jboolean] -> IO ();
	vmSetBooleanArrayRegion array start list	=
		convertSetRegion rawSetBooleanArrayRegion array start (fmap toRawBoolean list);
	vmSetByteArrayRegion	:: (?jvmenv :: VMEnv) =>
	 VMRef -> Jint -> [Jbyte] -> IO ();
	vmSetByteArrayRegion	= convertSetRegion rawSetByteArrayRegion;
	vmSetCharArrayRegion	:: (?jvmenv :: VMEnv) =>
	 VMRef -> Jint -> [Jchar] -> IO ();
	vmSetCharArrayRegion	= convertSetRegion rawSetCharArrayRegion;
	vmSetShortArrayRegion	:: (?jvmenv :: VMEnv) =>
	 VMRef -> Jint -> [Jshort] -> IO ();
	vmSetShortArrayRegion	= convertSetRegion rawSetShortArrayRegion;
	vmSetIntArrayRegion		:: (?jvmenv :: VMEnv) =>
	 VMRef -> Jint -> [Jint] -> IO ();
	vmSetIntArrayRegion		= convertSetRegion rawSetIntArrayRegion;
	vmSetLongArrayRegion	:: (?jvmenv :: VMEnv) =>
	 VMRef -> Jint -> [Jlong] -> IO ();
	vmSetLongArrayRegion	= convertSetRegion rawSetLongArrayRegion;
	vmSetFloatArrayRegion	:: (?jvmenv :: VMEnv) =>
	 VMRef -> Jint -> [Jfloat] -> IO ();
	vmSetFloatArrayRegion	= convertSetRegion rawSetFloatArrayRegion;
	vmSetDoubleArrayRegion	:: (?jvmenv :: VMEnv) =>
	 VMRef -> Jint -> [Jdouble] -> IO ();
	vmSetDoubleArrayRegion	= convertSetRegion rawSetDoubleArrayRegion;

	vmSetRefArrayElement	:: (?jvmenv :: VMEnv) =>
	 VMRef -> Jint -> VMRef -> IO ();
	vmSetRefArrayElement array i elem = withVMRefRawGlobal array (\rawArray ->
		withVMRefRawGlobal elem (\rawElem ->
			rawSetRefArrayElement ?jvmenv rawArray i rawElem
			));
	}
