-- This is written in Haskell.
{--
JVM-Bridge -- bridge from FP languages and others to the Java VM
Copyright (C) 2001 Ashley Yakeley <ashley@semantic.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--}

module VMCall where
	{
	import VMMethodID;
	import VMRef;
	import VMTypes;
	import RawLayer;
	import JavaLayer;

	convertOne :: (?jvmenv :: VMEnv) =>
	 (RawEnv -> RawGlobalRef -> mid -> RawValueList -> IO a) ->
		(VMRef -> mid -> VMValueList -> IO a);
	convertOne foo ref method vl = withVMRefRawGlobal ref (\rawRef -> foo ?jvmenv rawRef method vl);

	vmCallStaticVoidMethod		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticMethodID -> VMValueList -> IO Jvoid;
	vmCallStaticVoidMethod		= convertOne rawCallStaticVoidMethod;
	vmCallStaticBooleanMethod	:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticMethodID -> VMValueList -> IO Jboolean;
	vmCallStaticBooleanMethod ref method vl = do
		{
		rawB <- convertOne rawCallStaticBooleanMethod ref method vl;
		return (fromRawBoolean rawB);
		};
	vmCallStaticByteMethod		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticMethodID -> VMValueList -> IO Jbyte;
	vmCallStaticByteMethod		= convertOne rawCallStaticByteMethod;
	vmCallStaticCharMethod		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticMethodID -> VMValueList -> IO Jchar;
	vmCallStaticCharMethod		= convertOne rawCallStaticCharMethod;
	vmCallStaticShortMethod		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticMethodID -> VMValueList -> IO Jshort;
	vmCallStaticShortMethod		= convertOne rawCallStaticShortMethod;
	vmCallStaticIntMethod		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticMethodID -> VMValueList -> IO Jint;
	vmCallStaticIntMethod		= convertOne rawCallStaticIntMethod;
	vmCallStaticLongMethod		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticMethodID -> VMValueList -> IO Jlong;
	vmCallStaticLongMethod		= convertOne rawCallStaticLongMethod;
	vmCallStaticFloatMethod		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticMethodID -> VMValueList -> IO Jfloat;
	vmCallStaticFloatMethod		= convertOne rawCallStaticFloatMethod;
	vmCallStaticDoubleMethod	:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticMethodID -> VMValueList -> IO Jdouble;
	vmCallStaticDoubleMethod	= convertOne rawCallStaticDoubleMethod;
	vmCallStaticRefMethod		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticMethodID -> VMValueList -> IO VMRef;
	vmCallStaticRefMethod ref method vl = do
		{
		locRef <- convertOne rawCallStaticRefMethod ref method vl;
		makeVMRefFromLocal locRef;
		};

	vmCallVoidMethod	:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMMethodID -> VMValueList -> IO Jvoid;
	vmCallVoidMethod	= convertOne rawCallVoidMethod;
	vmCallBooleanMethod	:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMMethodID -> VMValueList -> IO Jboolean;
	vmCallBooleanMethod ref method vl = do
		{
		rawB <- convertOne rawCallBooleanMethod ref method vl;
		return (fromRawBoolean rawB);
		};
	vmCallByteMethod	:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMMethodID -> VMValueList -> IO Jbyte;
	vmCallByteMethod	= convertOne rawCallByteMethod;
	vmCallCharMethod	:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMMethodID -> VMValueList -> IO Jchar;
	vmCallCharMethod	= convertOne rawCallCharMethod;
	vmCallShortMethod	:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMMethodID -> VMValueList -> IO Jshort;
	vmCallShortMethod	= convertOne rawCallShortMethod;
	vmCallIntMethod		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMMethodID -> VMValueList -> IO Jint;
	vmCallIntMethod		= convertOne rawCallIntMethod;
	vmCallLongMethod	:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMMethodID -> VMValueList -> IO Jlong;
	vmCallLongMethod	= convertOne rawCallLongMethod;
	vmCallFloatMethod	:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMMethodID -> VMValueList -> IO Jfloat;
	vmCallFloatMethod	= convertOne rawCallFloatMethod;
	vmCallDoubleMethod	:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMMethodID -> VMValueList -> IO Jdouble;
	vmCallDoubleMethod	= convertOne rawCallDoubleMethod;
	vmCallRefMethod		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMMethodID -> VMValueList -> IO VMRef;
	vmCallRefMethod ref method vl = do
		{
		locRef <- convertOne rawCallRefMethod ref method vl;
		makeVMRefFromLocal locRef;
		};

	convertTwo :: (?jvmenv :: VMEnv) =>
	 (RawEnv -> RawGlobalRef -> RawGlobalRef -> RawMethodID -> RawValueList -> IO a) ->
		(VMRef -> VMRef -> VMMethodID -> VMValueList -> IO a);
	convertTwo foo obj cls method vl = withVMRefRawGlobal obj (\rawObj -> 
		withVMRefRawGlobal cls (\rawCls -> foo ?jvmenv rawObj rawCls method vl));

	vmCallNonvirtualVoidMethod		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMRef -> VMMethodID -> VMValueList -> IO Jvoid;
	vmCallNonvirtualVoidMethod		= convertTwo rawCallNonvirtualVoidMethod;
	vmCallNonvirtualBooleanMethod	:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMRef -> VMMethodID -> VMValueList -> IO Jboolean;
	vmCallNonvirtualBooleanMethod cls obj method vl = do
		{
		rawB <- convertTwo rawCallNonvirtualBooleanMethod cls obj method vl;
		return (fromRawBoolean rawB);
		};
	vmCallNonvirtualByteMethod		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMRef -> VMMethodID -> VMValueList -> IO Jbyte;
	vmCallNonvirtualByteMethod		= convertTwo rawCallNonvirtualByteMethod;
	vmCallNonvirtualCharMethod		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMRef -> VMMethodID -> VMValueList -> IO Jchar;
	vmCallNonvirtualCharMethod		= convertTwo rawCallNonvirtualCharMethod;
	vmCallNonvirtualShortMethod		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMRef -> VMMethodID -> VMValueList -> IO Jshort;
	vmCallNonvirtualShortMethod		= convertTwo rawCallNonvirtualShortMethod;
	vmCallNonvirtualIntMethod		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMRef -> VMMethodID -> VMValueList -> IO Jint;
	vmCallNonvirtualIntMethod		= convertTwo rawCallNonvirtualIntMethod;
	vmCallNonvirtualLongMethod		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMRef -> VMMethodID -> VMValueList -> IO Jlong;
	vmCallNonvirtualLongMethod		= convertTwo rawCallNonvirtualLongMethod;
	vmCallNonvirtualFloatMethod		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMRef -> VMMethodID -> VMValueList -> IO Jfloat;
	vmCallNonvirtualFloatMethod		= convertTwo rawCallNonvirtualFloatMethod;
	vmCallNonvirtualDoubleMethod	:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMRef -> VMMethodID -> VMValueList -> IO Jdouble;
	vmCallNonvirtualDoubleMethod	= convertTwo rawCallNonvirtualDoubleMethod;
	vmCallNonvirtualRefMethod		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMRef -> VMMethodID -> VMValueList -> IO VMRef;
	vmCallNonvirtualRefMethod cls obj method vl = do
		{
		locRef <- convertTwo rawCallNonvirtualRefMethod cls obj method vl;
		makeVMRefFromLocal locRef;
		};
	}
