-- This is written in Haskell.
{--
JVM-Bridge -- bridge from FP languages and others to the Java VM
Copyright (C) 2001 Ashley Yakeley <ashley@semantic.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--}

module VMCallback
	(
	OpaqueAddress,
	executeFunctionClassName,
	vmStartExecuteFunction,
	vmMakeVoidCallback,
	vmMakeBooleanCallback,
	vmMakeByteCallback,
	vmMakeCharCallback,
	vmMakeShortCallback,
	vmMakeIntCallback,
	vmMakeLongCallback,
	vmMakeFloatCallback,
	vmMakeDoubleCallback,
	vmMakeRefCallback
	,vmDefineClass,vmLoadClass,vmDefineCallbackClass
	) where
	{
	import VMRef;
	import VMStringPtr;
	import VMTypes;
	import RawLayer;
	import JCFLayer;
	import JavaLayer;
	import Foreign;

	vmStartExecuteFunction	:: (?jvmenv :: VMEnv) => IO ();
	vmStartExecuteFunction = rawStartExecuteFunction ?jvmenv;

	convertCallback :: ((RawEnv -> RawValueList -> IO a) -> IO (FunPtr ())) ->
		((?jvmenv :: VMEnv) => VMValueList -> IO a) -> IO OpaqueAddress;
	convertCallback rawF foo = do
		{
		ptr <- rawF (\env -> let {?jvmenv=env;} in foo);
		rawConvertCallback ptr;
		};

	vmMakeVoidCallback		:: ((?jvmenv :: VMEnv) =>
	 VMValueList -> IO Jvoid) -> IO OpaqueAddress;
	vmMakeVoidCallback		= convertCallback rawMakeVoidCallback;

	vmMakeBooleanCallback	:: ((?jvmenv :: VMEnv) =>
	 VMValueList -> IO Jboolean) -> IO OpaqueAddress;
	vmMakeBooleanCallback foo	= convertCallback rawMakeBooleanCallback (\vl -> do
		{
		b <- foo vl;
		return (toRawBoolean b);
		});

	vmMakeByteCallback		:: ((?jvmenv :: VMEnv) =>
	 VMValueList -> IO Jbyte) -> IO OpaqueAddress;
	vmMakeByteCallback		= convertCallback rawMakeByteCallback;

	vmMakeCharCallback		:: ((?jvmenv :: VMEnv) =>
	 VMValueList -> IO Jchar) -> IO OpaqueAddress;
	vmMakeCharCallback		= convertCallback rawMakeCharCallback;

	vmMakeShortCallback		:: ((?jvmenv :: VMEnv) =>
	 VMValueList -> IO Jshort) -> IO OpaqueAddress;
	vmMakeShortCallback		= convertCallback rawMakeShortCallback;

	vmMakeIntCallback		:: ((?jvmenv :: VMEnv) =>
	 VMValueList -> IO Jint) -> IO OpaqueAddress;
	vmMakeIntCallback		= convertCallback rawMakeIntCallback;

	vmMakeLongCallback		:: ((?jvmenv :: VMEnv) =>
	 VMValueList -> IO Jlong) -> IO OpaqueAddress;
	vmMakeLongCallback		= convertCallback rawMakeLongCallback;

	vmMakeFloatCallback		:: ((?jvmenv :: VMEnv) =>
	 VMValueList -> IO Jfloat) -> IO OpaqueAddress;
	vmMakeFloatCallback		= convertCallback rawMakeFloatCallback;

	vmMakeDoubleCallback	:: ((?jvmenv :: VMEnv) =>
	 VMValueList -> IO Jdouble) -> IO OpaqueAddress;
	vmMakeDoubleCallback	= convertCallback rawMakeDoubleCallback;

	vmMakeRefCallback		:: ((?jvmenv :: VMEnv) =>
	 VMValueList -> IO VMRef) -> IO OpaqueAddress;
	vmMakeRefCallback foo	= convertCallback rawMakeObjectCallback (\vl -> do
		{
		ref <- foo vl;
		withVMRefRawGlobal ref return;
		});

	vmDefineClass :: (?jvmenv :: VMEnv) =>
	 ClassName -> VMRef -> [Word8] -> IO VMRef;
	vmDefineClass name loader cf = do
		{
		loc <- withJString name (\pname ->
			withVMRefRawGlobal loader (\rloader ->
				withArray cf (\pcf ->
					rawDefineClass ?jvmenv pname rloader ((castPtr :: Ptr Word8 -> Ptr RawByte) pcf) (fromIntegral (length cf))
			)));
		makeVMRefFromLocal loc;
		};

	vmLoadClass :: (?jvmenv :: VMEnv) =>
	 VMRef -> ClassDefinition -> IO VMRef;
	vmLoadClass loader cls = vmDefineClass (classDefName cls) loader (jcfWrite cls);

	vmDefineCallbackClass :: (?jvmenv :: VMEnv) =>
	 VMRef -> ClassName -> ClassName -> [ClassName] -> [ValueType] -> [MethodNameType] -> IO VMRef;
	vmDefineCallbackClass loader classname superclassname interfaces superclassconstructor methods = 
		vmDefineClass classname loader (jcfWrite (makeDeferredCallbackClass classname superclassname interfaces superclassconstructor methods));
	}
