-- This is written in Haskell.
{--
JVM-Bridge -- bridge from FP languages and others to the Java VM
Copyright (C) 2001 Ashley Yakeley <ashley@semantic.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--}

module VMFieldAccess where
	{
	import VMFieldID;
	import VMRef;
	import VMTypes;
	import RawLayer;
	import JavaLayer;

	convertGet :: (?jvmenv :: VMEnv) => (RawEnv -> RawGlobalRef -> fid -> IO a) ->
		(VMRef -> fid -> IO a);
	convertGet foo ref field = withVMRefRawGlobal ref (\rawRef -> foo ?jvmenv rawRef field);

	convertSet :: (?jvmenv :: VMEnv) => (RawEnv -> RawGlobalRef -> fid -> a -> IO ()) ->
		(VMRef -> fid -> a -> IO ());
	convertSet foo ref field val = withVMRefRawGlobal ref (\rawRef -> foo ?jvmenv rawRef field val);

	vmGetStaticBooleanField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticFieldID -> IO Jboolean;
	vmGetStaticBooleanField ref field = do
		{
		rawB <- convertGet rawGetStaticBooleanField ref field;
		return (fromRawBoolean rawB);
		};
	vmGetStaticByteField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticFieldID -> IO Jbyte;
	vmGetStaticByteField		= convertGet rawGetStaticByteField;
	vmGetStaticCharField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticFieldID -> IO Jchar;
	vmGetStaticCharField		= convertGet rawGetStaticCharField;
	vmGetStaticShortField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticFieldID -> IO Jshort;
	vmGetStaticShortField		= convertGet rawGetStaticShortField;
	vmGetStaticIntField			:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticFieldID -> IO Jint;
	vmGetStaticIntField			= convertGet rawGetStaticIntField;
	vmGetStaticLongField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticFieldID -> IO Jlong;
	vmGetStaticLongField		= convertGet rawGetStaticLongField;
	vmGetStaticFloatField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticFieldID -> IO Jfloat;
	vmGetStaticFloatField		= convertGet rawGetStaticFloatField;
	vmGetStaticDoubleField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticFieldID -> IO Jdouble;
	vmGetStaticDoubleField		= convertGet rawGetStaticDoubleField;
	vmGetStaticRefField			:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticFieldID -> IO VMRef;
	vmGetStaticRefField ref field = do
		{
		locRef <- convertGet rawGetStaticRefField ref field;
		makeVMRefFromLocal locRef;
		};

	vmSetStaticBooleanField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticFieldID -> Jboolean -> IO ();
	vmSetStaticBooleanField ref field val =
		convertSet rawSetStaticBooleanField ref field (toRawBoolean val);
	vmSetStaticByteField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticFieldID -> Jbyte -> IO ();
	vmSetStaticByteField		= convertSet rawSetStaticByteField;
	vmSetStaticCharField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticFieldID -> Jchar -> IO ();
	vmSetStaticCharField		= convertSet rawSetStaticCharField;
	vmSetStaticShortField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticFieldID -> Jshort -> IO ();
	vmSetStaticShortField		= convertSet rawSetStaticShortField;
	vmSetStaticIntField			:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticFieldID -> Jint -> IO ();
	vmSetStaticIntField			= convertSet rawSetStaticIntField;
	vmSetStaticLongField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticFieldID -> Jlong -> IO ();
	vmSetStaticLongField		= convertSet rawSetStaticLongField;
	vmSetStaticFloatField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticFieldID -> Jfloat -> IO ();
	vmSetStaticFloatField		= convertSet rawSetStaticFloatField;
	vmSetStaticDoubleField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticFieldID -> Jdouble -> IO ();
	vmSetStaticDoubleField		= convertSet rawSetStaticDoubleField;
	vmSetStaticRefField			:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMStaticFieldID -> VMRef -> IO ();
	vmSetStaticRefField ref field val = withVMRefRawGlobal val (\rawVal -> 
		convertSet rawSetStaticRefField ref field rawVal
		);

	vmGetBooleanField	:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMFieldID -> IO Jboolean;
	vmGetBooleanField ref field = do
		{
		rawB <- convertGet rawGetBooleanField ref field;
		return (fromRawBoolean rawB);
		};
	vmGetByteField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMFieldID -> IO Jbyte;
	vmGetByteField		= convertGet rawGetByteField;
	vmGetCharField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMFieldID -> IO Jchar;
	vmGetCharField		= convertGet rawGetCharField;
	vmGetShortField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMFieldID -> IO Jshort;
	vmGetShortField		= convertGet rawGetShortField;
	vmGetIntField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMFieldID -> IO Jint;
	vmGetIntField		= convertGet rawGetIntField;
	vmGetLongField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMFieldID -> IO Jlong;
	vmGetLongField		= convertGet rawGetLongField;
	vmGetFloatField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMFieldID -> IO Jfloat;
	vmGetFloatField		= convertGet rawGetFloatField;
	vmGetDoubleField	:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMFieldID -> IO Jdouble;
	vmGetDoubleField	= convertGet rawGetDoubleField;
	vmGetRefField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMFieldID -> IO VMRef;
	vmGetRefField ref field = do
		{
		locRef <- convertGet rawGetRefField ref field;
		makeVMRefFromLocal locRef;
		};

	vmSetBooleanField	:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMFieldID -> Jboolean -> IO ();
	vmSetBooleanField ref field val =
		convertSet rawSetBooleanField ref field (toRawBoolean val);
	vmSetByteField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMFieldID -> Jbyte -> IO ();
	vmSetByteField		= convertSet rawSetByteField;
	vmSetCharField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMFieldID -> Jchar -> IO ();
	vmSetCharField		= convertSet rawSetCharField;
	vmSetShortField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMFieldID -> Jshort -> IO ();
	vmSetShortField		= convertSet rawSetShortField;
	vmSetIntField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMFieldID -> Jint -> IO ();
	vmSetIntField		= convertSet rawSetIntField;
	vmSetLongField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMFieldID -> Jlong -> IO ();
	vmSetLongField		= convertSet rawSetLongField;
	vmSetFloatField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMFieldID -> Jfloat -> IO ();
	vmSetFloatField		= convertSet rawSetFloatField;
	vmSetDoubleField	:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMFieldID -> Jdouble -> IO ();
	vmSetDoubleField	= convertSet rawSetDoubleField;
	vmSetRefField		:: (?jvmenv :: VMEnv) =>
	 VMRef -> VMFieldID -> VMRef -> IO ();
	vmSetRefField ref field val = withVMRefRawGlobal val (\rawVal -> 
		convertSet rawSetRefField ref field rawVal
		);
	}
