-- This is written in Haskell.
{--
JVM-Bridge -- bridge from FP languages and others to the Java VM
Copyright (C) 2001 Ashley Yakeley <ashley@semantic.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--}

module VMMethodID where
	{
	import VMStringPtr;
	import VMRef;
	import VMTypes;
	import RawLayer;
	import JavaLayer;
	import BasicLayer;

	type VMStaticMethodID	= RawStaticMethodID;
	type VMMethodID			= RawMethodID;
	
	instance (Monad m) => MonadHasNothing m VMStaticMethodID where
		{
		getNothing = return nothing;
		getIsNothing a = return (isNothing a);
		};
	
	instance HasNothing VMStaticMethodID where
		{
		nothing = (MkRawStaticMethodID nothing);
		isNothing (MkRawStaticMethodID a) = isNothing a;
		};
	
	instance (Monad m) => MonadHasNothing m VMMethodID where
		{
		getNothing = return nothing;
		getIsNothing a = return (isNothing a);
		};
	
	instance HasNothing VMMethodID where
		{
		nothing = (MkRawMethodID nothing);
		isNothing (MkRawMethodID a) = isNothing a;
		};
	
	vmGetStaticMethodID :: (?jvmenv :: VMEnv) =>
	 VMRef -> MemberName -> FunctionType -> IO VMStaticMethodID;
	vmGetStaticMethodID jClass name ftype =
		withJString name (\pname ->
			withJString (functionTypeSig ftype) (\psig ->
				withVMRefRawGlobal jClass (\rawClass ->
					rawGetStaticMethodID ?jvmenv rawClass pname psig)));

	vmGetMethodID :: (?jvmenv :: VMEnv) =>
	 VMRef -> MemberName -> FunctionType -> IO VMMethodID;
	vmGetMethodID jClass name ftype =
		withJString name (\pname ->
			withJString (functionTypeSig ftype) (\psig ->
				withVMRefRawGlobal jClass (\rawClass ->
					rawGetMethodID ?jvmenv rawClass pname psig)));
	}
