-- This is written in Haskell.
{--
JVM-Bridge -- bridge from FP languages and others to the Java VM
Copyright (C) 2001 Ashley Yakeley <ashley@semantic.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--}

module VMThrowable where
	{
	import VMRef;
	import VMTypes;
	import RawLayer;
	import JavaLayer;

	reportFailedJavaException :: IO a;
	reportFailedJavaException = fail "couldn't throw Java VM exception";

	vmThrow :: (?jvmenv :: VMEnv) =>
	 VMRef -> IO a;
	vmThrow th = do
		{
		result <- withVMRefRawGlobal th (\rawTh -> rawThrowException ?jvmenv rawTh);
		if (result == 0) then return undefined else reportFailedJavaException;
		};

	vmGetClearException :: (?jvmenv :: VMEnv) =>
	 IO VMRef;
	vmGetClearException = do
		{
		x <- rawGetClearException ?jvmenv;
		makeVMRefFromLocal x; 
		};

	vmExceptionPending :: (?jvmenv :: VMEnv) =>
	 IO Jboolean;
	vmExceptionPending = do
		{
		is <- rawExceptionPending ?jvmenv;
		return (fromRawBoolean is); 
		};

	reportJavaException :: IO a;
	reportJavaException = ioError (userError "pending VM exception");

	vmExceptionCheck :: (?jvmenv :: VMEnv) =>
	 IO ();
	vmExceptionCheck = do
		{
		ex <- vmExceptionPending;
--	putStrLn ("vmExceptionPending (Word8): " ++ (show (ex::Word8)));
		if ex then reportJavaException else (return ());
		};
	
	vmCallChecked :: (?jvmenv :: VMEnv) =>
	 ((?jvmenv :: VMEnv) => IO a) -> IO a;
	vmCallChecked foo = do
		{
		result <- foo;
		vmExceptionCheck;
		return result;
		};
	}
