-- This is written in Haskell.
{--
JVM-Bridge -- bridge from FP languages and others to the Java VM
Copyright (C) 2001 Ashley Yakeley <ashley@semantic.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--}

module VMValueList where
	{
	import VMRef;
	import VMTypes;
	import RawLayer;
	import JavaLayer;
	import Word;

	vmCreateValueList :: IO VMValueList;
	vmCreateValueList = rawCreateValueList;
	
	vmDestroyValueList :: VMValueList -> IO ();
	vmDestroyValueList = rawDestroyValueList;
	
	vmGetNthValueAsBoolean	:: VMValueList -> Word16 -> IO Jboolean;
	vmGetNthValueAsBoolean vl i	= do
		{
		raw <- rawGetNthValueAsBoolean vl i;
		return (fromRawBoolean raw);
		};
	vmGetNthValueAsByte		:: VMValueList -> Word16 -> IO Jbyte;
	vmGetNthValueAsByte		= rawGetNthValueAsByte;
	vmGetNthValueAsChar		:: VMValueList -> Word16 -> IO Jchar;
	vmGetNthValueAsChar		= rawGetNthValueAsChar;
	vmGetNthValueAsShort	:: VMValueList -> Word16 -> IO Jshort;
	vmGetNthValueAsShort	= rawGetNthValueAsShort;
	vmGetNthValueAsInt		:: VMValueList -> Word16 -> IO Jint;
	vmGetNthValueAsInt		= rawGetNthValueAsInt;
	vmGetNthValueAsLong		:: VMValueList -> Word16 -> IO Jlong;
	vmGetNthValueAsLong		= rawGetNthValueAsLong;
	vmGetNthValueAsFloat	:: VMValueList -> Word16 -> IO Jfloat;
	vmGetNthValueAsFloat	= rawGetNthValueAsFloat;
	vmGetNthValueAsDouble	:: VMValueList -> Word16 -> IO Jdouble;
	vmGetNthValueAsDouble	= rawGetNthValueAsDouble;
	vmGetNthValueAsRef		:: (?jvmenv :: VMEnv) =>
	 VMValueList -> Word16 -> IO VMRef;
	vmGetNthValueAsRef vl i	= do
		{
		rawGlobal <- rawGetNthValueAsRef vl i;
		makeVMRefFromGlobal rawGlobal;
		};

	vmAddBooleanToValueList :: VMValueList -> Jboolean -> IO ();
	vmAddBooleanToValueList vl x = rawAddBooleanToValueList vl (toRawBoolean x);
	vmAddByteToValueList	:: VMValueList -> Jbyte -> IO ();
	vmAddByteToValueList	= rawAddByteToValueList;
	vmAddCharToValueList	:: VMValueList -> Jchar -> IO ();
	vmAddCharToValueList	= rawAddCharToValueList;
	vmAddShortToValueList	:: VMValueList -> Jshort -> IO ();
	vmAddShortToValueList	= rawAddShortToValueList;
	vmAddIntToValueList		:: VMValueList -> Jint -> IO ();
	vmAddIntToValueList		= rawAddIntToValueList;
	vmAddLongToValueList	:: VMValueList -> Jlong -> IO ();
	vmAddLongToValueList	= rawAddLongToValueList;
	vmAddFloatToValueList	:: VMValueList -> Jfloat -> IO ();
	vmAddFloatToValueList	= rawAddFloatToValueList;
	vmAddDoubleToValueList	:: VMValueList -> Jdouble -> IO ();
	vmAddDoubleToValueList	= rawAddDoubleToValueList;
	vmAddRefToValueList		:: VMValueList -> VMRef -> IO ();
	vmAddRefToValueList vl ref = withVMRefRawGlobal ref (\rawRef -> rawAddRefToValueList vl rawRef);
	}
